
package tip.analysis;

import tip.node.*;

/**
 * {@code AnswerAdapter} implements the default behaviour of the {@link Answer}
 * interface. All calls return {@code null} by default.
 *
 * @param <A> the type of the answers
 *
 * @author Johnni Winther, jw@brics.dk
 */
public class AnswerAdapter<A> implements Answer<A> {

	/**
	 * Returns the default answer for a {@link Node}.
	 * @param node the calling {@link Node}
	 * @return the default answer for a {@link Node}
	 */
	public A defaultNode(Node node) {
		return null;
	}
	
	/**
	 * Returns the default answer for a {@link Token}. The call is deferred to
	 * {@link AnswerAdapter#defaultNode(Node)}.
	 * @param token the calling {@link Token}
	 * @return the default answer for a {@link Token}
	 */
	public A defaultToken(Token token) {
		return defaultNode(token);
	}
	
	/**
	 * Returns the default answer for a {@link Start} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link Start} node
	 * @return the default answer for a {@link Start} node
	 */
    public A caseStart(Start node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link AProgram} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPProgram(PProgram)}.
	 * @param node the calling {@link AProgram} node
	 * @return the default answer for a {@link AProgram} node
	 */
	public A caseAProgram(AProgram node) {
		return defaultPProgram(node);
	}
	
	/**
	 * Returns the default answer for a {@link AFuncDecl} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPDecl(PDecl)}.
	 * @param node the calling {@link AFuncDecl} node
	 * @return the default answer for a {@link AFuncDecl} node
	 */
	public A caseAFuncDecl(AFuncDecl node) {
		return defaultPDecl(node);
	}
	
	/**
	 * Returns the default answer for a {@link ALocalDecl} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPDecl(PDecl)}.
	 * @param node the calling {@link ALocalDecl} node
	 * @return the default answer for a {@link ALocalDecl} node
	 */
	public A caseALocalDecl(ALocalDecl node) {
		return defaultPDecl(node);
	}
	
	/**
	 * Returns the default answer for a {@link ABody} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBody(PBody)}.
	 * @param node the calling {@link ABody} node
	 * @return the default answer for a {@link ABody} node
	 */
	public A caseABody(ABody node) {
		return defaultPBody(node);
	}
	
	/**
	 * Returns the default answer for a {@link ABlock} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBlock(PBlock)}.
	 * @param node the calling {@link ABlock} node
	 * @return the default answer for a {@link ABlock} node
	 */
	public A caseABlock(ABlock node) {
		return defaultPBlock(node);
	}
	
	/**
	 * Returns the default answer for a {@link APlusBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link APlusBinop} node
	 * @return the default answer for a {@link APlusBinop} node
	 */
	public A caseAPlusBinop(APlusBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link AMinusBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link AMinusBinop} node
	 * @return the default answer for a {@link AMinusBinop} node
	 */
	public A caseAMinusBinop(AMinusBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link ATimesBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link ATimesBinop} node
	 * @return the default answer for a {@link ATimesBinop} node
	 */
	public A caseATimesBinop(ATimesBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link ADivideBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link ADivideBinop} node
	 * @return the default answer for a {@link ADivideBinop} node
	 */
	public A caseADivideBinop(ADivideBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link AEqBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link AEqBinop} node
	 * @return the default answer for a {@link AEqBinop} node
	 */
	public A caseAEqBinop(AEqBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link AGtBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPBinop(PBinop)}.
	 * @param node the calling {@link AGtBinop} node
	 * @return the default answer for a {@link AGtBinop} node
	 */
	public A caseAGtBinop(AGtBinop node) {
		return defaultPBinop(node);
	}
	
	/**
	 * Returns the default answer for a {@link ABinopExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link ABinopExp} node
	 * @return the default answer for a {@link ABinopExp} node
	 */
	public A caseABinopExp(ABinopExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link APointerExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link APointerExp} node
	 * @return the default answer for a {@link APointerExp} node
	 */
	public A caseAPointerExp(APointerExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link ADerefExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link ADerefExp} node
	 * @return the default answer for a {@link ADerefExp} node
	 */
	public A caseADerefExp(ADerefExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AIntConstExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AIntConstExp} node
	 * @return the default answer for a {@link AIntConstExp} node
	 */
	public A caseAIntConstExp(AIntConstExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AFunctionCallExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AFunctionCallExp} node
	 * @return the default answer for a {@link AFunctionCallExp} node
	 */
	public A caseAFunctionCallExp(AFunctionCallExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AFunctionPtrCallExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AFunctionPtrCallExp} node
	 * @return the default answer for a {@link AFunctionPtrCallExp} node
	 */
	public A caseAFunctionPtrCallExp(AFunctionPtrCallExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AVarExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AVarExp} node
	 * @return the default answer for a {@link AVarExp} node
	 */
	public A caseAVarExp(AVarExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AInputExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AInputExp} node
	 * @return the default answer for a {@link AInputExp} node
	 */
	public A caseAInputExp(AInputExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AMallocExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link AMallocExp} node
	 * @return the default answer for a {@link AMallocExp} node
	 */
	public A caseAMallocExp(AMallocExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link ANullExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPExp(PExp)}.
	 * @param node the calling {@link ANullExp} node
	 * @return the default answer for a {@link ANullExp} node
	 */
	public A caseANullExp(ANullExp node) {
		return defaultPExp(node);
	}
	
	/**
	 * Returns the default answer for a {@link AIfThenStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AIfThenStm} node
	 * @return the default answer for a {@link AIfThenStm} node
	 */
	public A caseAIfThenStm(AIfThenStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AIfThenElseStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AIfThenElseStm} node
	 * @return the default answer for a {@link AIfThenElseStm} node
	 */
	public A caseAIfThenElseStm(AIfThenElseStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AWhileStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AWhileStm} node
	 * @return the default answer for a {@link AWhileStm} node
	 */
	public A caseAWhileStm(AWhileStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link ADeclStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link ADeclStm} node
	 * @return the default answer for a {@link ADeclStm} node
	 */
	public A caseADeclStm(ADeclStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AReturnStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AReturnStm} node
	 * @return the default answer for a {@link AReturnStm} node
	 */
	public A caseAReturnStm(AReturnStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AOutputStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AOutputStm} node
	 * @return the default answer for a {@link AOutputStm} node
	 */
	public A caseAOutputStm(AOutputStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AAssignVarStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AAssignVarStm} node
	 * @return the default answer for a {@link AAssignVarStm} node
	 */
	public A caseAAssignVarStm(AAssignVarStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link AAssignPtrStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link AAssignPtrStm} node
	 * @return the default answer for a {@link AAssignPtrStm} node
	 */
	public A caseAAssignPtrStm(AAssignPtrStm node) {
		return defaultPStm(node);
	}
	
	/**
	 * Returns the default answer for a {@link ABlockStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultPStm(PStm)}.
	 * @param node the calling {@link ABlockStm} node
	 * @return the default answer for a {@link ABlockStm} node
	 */
	public A caseABlockStm(ABlockStm node) {
		return defaultPStm(node);
	}
	

	/**
	 * Returns the default answer for a {@link TWhiteSpace} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TWhiteSpace} node
	 * @return the default answer for a {@link TWhiteSpace} node
	 */
	public A caseTWhiteSpace(TWhiteSpace node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TCommentSingle} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TCommentSingle} node
	 * @return the default answer for a {@link TCommentSingle} node
	 */
	public A caseTCommentSingle(TCommentSingle node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TCommentBlock} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TCommentBlock} node
	 * @return the default answer for a {@link TCommentBlock} node
	 */
	public A caseTCommentBlock(TCommentBlock node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TVar} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TVar} node
	 * @return the default answer for a {@link TVar} node
	 */
	public A caseTVar(TVar node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TInput} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TInput} node
	 * @return the default answer for a {@link TInput} node
	 */
	public A caseTInput(TInput node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TOutput} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TOutput} node
	 * @return the default answer for a {@link TOutput} node
	 */
	public A caseTOutput(TOutput node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TIf} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TIf} node
	 * @return the default answer for a {@link TIf} node
	 */
	public A caseTIf(TIf node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TElse} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TElse} node
	 * @return the default answer for a {@link TElse} node
	 */
	public A caseTElse(TElse node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TWhile} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TWhile} node
	 * @return the default answer for a {@link TWhile} node
	 */
	public A caseTWhile(TWhile node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TReturn} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TReturn} node
	 * @return the default answer for a {@link TReturn} node
	 */
	public A caseTReturn(TReturn node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TMalloc} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TMalloc} node
	 * @return the default answer for a {@link TMalloc} node
	 */
	public A caseTMalloc(TMalloc node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TNull} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TNull} node
	 * @return the default answer for a {@link TNull} node
	 */
	public A caseTNull(TNull node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TLPar} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TLPar} node
	 * @return the default answer for a {@link TLPar} node
	 */
	public A caseTLPar(TLPar node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TRPar} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TRPar} node
	 * @return the default answer for a {@link TRPar} node
	 */
	public A caseTRPar(TRPar node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TLBrace} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TLBrace} node
	 * @return the default answer for a {@link TLBrace} node
	 */
	public A caseTLBrace(TLBrace node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TRBrace} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TRBrace} node
	 * @return the default answer for a {@link TRBrace} node
	 */
	public A caseTRBrace(TRBrace node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TSemicolon} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TSemicolon} node
	 * @return the default answer for a {@link TSemicolon} node
	 */
	public A caseTSemicolon(TSemicolon node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TComma} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TComma} node
	 * @return the default answer for a {@link TComma} node
	 */
	public A caseTComma(TComma node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TAssign} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TAssign} node
	 * @return the default answer for a {@link TAssign} node
	 */
	public A caseTAssign(TAssign node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TPlus} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TPlus} node
	 * @return the default answer for a {@link TPlus} node
	 */
	public A caseTPlus(TPlus node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TMinus} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TMinus} node
	 * @return the default answer for a {@link TMinus} node
	 */
	public A caseTMinus(TMinus node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TStar} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TStar} node
	 * @return the default answer for a {@link TStar} node
	 */
	public A caseTStar(TStar node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TDiv} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TDiv} node
	 * @return the default answer for a {@link TDiv} node
	 */
	public A caseTDiv(TDiv node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TAnd} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TAnd} node
	 * @return the default answer for a {@link TAnd} node
	 */
	public A caseTAnd(TAnd node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TGt} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TGt} node
	 * @return the default answer for a {@link TGt} node
	 */
	public A caseTGt(TGt node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TEq} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TEq} node
	 * @return the default answer for a {@link TEq} node
	 */
	public A caseTEq(TEq node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TIntegerConst} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TIntegerConst} node
	 * @return the default answer for a {@link TIntegerConst} node
	 */
	public A caseTIntegerConst(TIntegerConst node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link TIdentifier} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link TIdentifier} node
	 * @return the default answer for a {@link TIdentifier} node
	 */
	public A caseTIdentifier(TIdentifier node) {
		return defaultToken(node);
	}
	
	/**
	 * Returns the default answer for a {@link PProgram} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PProgram} node
	 * @return the default answer for a {@link PProgram} node
	 */
	public A defaultPProgram(PProgram node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PDecl} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PDecl} node
	 * @return the default answer for a {@link PDecl} node
	 */
	public A defaultPDecl(PDecl node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PBody} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PBody} node
	 * @return the default answer for a {@link PBody} node
	 */
	public A defaultPBody(PBody node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PBlock} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PBlock} node
	 * @return the default answer for a {@link PBlock} node
	 */
	public A defaultPBlock(PBlock node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PBinop} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PBinop} node
	 * @return the default answer for a {@link PBinop} node
	 */
	public A defaultPBinop(PBinop node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PExp} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PExp} node
	 * @return the default answer for a {@link PExp} node
	 */
	public A defaultPExp(PExp node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link PStm} node. The call is deferred
	 * to {@link AnswerAdapter#defaultNode(Node)}.
	 * @param node the calling {@link PStm} node
	 * @return the default answer for a {@link PStm} node
	 */
	public A defaultPStm(PStm node) {
		return defaultNode(node);
	}
	
	/**
	 * Returns the default answer for a {@link EOF} node. The call is deferred
	 * to {@link AnswerAdapter#defaultToken(Token)}.
	 * @param node the calling {@link EOF} node
	 * @return the default answer for a {@link EOF} node
	 */
    public A caseEOF(EOF node) {
		return defaultToken(node);
	}

}
