
package tip.analysis;

import tip.node.*;

/**
 * {@code QuestionAdapter} implements the default behaviour of the {@link Question}
 * interface.
 *
 * @param <Q> the type of the questions
 *
 * @author Johnni Winther, jw@brics.dk
 */
public class QuestionAdapter<Q> implements Question<Q> {

	/**
	 * Default action for a question to a {@link Node}.
	 * @param node the calling {@link Node}
	 * @param question the provided question
	 */
	public void defaultNode(Node node, Q question) {
		// empty body
	}
	
	/**
	 * Default action for a question to a {@link Token}. 
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param token the calling {@link Token}
	 * @param question the provided question
	 */
	public void defaultToken(Token token, Q question) {
		defaultNode(token, question);
	}
	
	/**
	 * Action for applying a question to a {@link Start} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link Start} node
	 * @param question the provided question
	 */
    public void caseStart(Start node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AProgram} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPProgram(PProgram,Object)}.
	 * @param node the calling {@link AProgram} node
	 * @param question the provided question
	 */
	public void caseAProgram(AProgram node, Q question) {
		defaultPProgram(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AFuncDecl} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPDecl(PDecl,Object)}.
	 * @param node the calling {@link AFuncDecl} node
	 * @param question the provided question
	 */
	public void caseAFuncDecl(AFuncDecl node, Q question) {
		defaultPDecl(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ALocalDecl} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPDecl(PDecl,Object)}.
	 * @param node the calling {@link ALocalDecl} node
	 * @param question the provided question
	 */
	public void caseALocalDecl(ALocalDecl node, Q question) {
		defaultPDecl(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ABody} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBody(PBody,Object)}.
	 * @param node the calling {@link ABody} node
	 * @param question the provided question
	 */
	public void caseABody(ABody node, Q question) {
		defaultPBody(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ABlock} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBlock(PBlock,Object)}.
	 * @param node the calling {@link ABlock} node
	 * @param question the provided question
	 */
	public void caseABlock(ABlock node, Q question) {
		defaultPBlock(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link APlusBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link APlusBinop} node
	 * @param question the provided question
	 */
	public void caseAPlusBinop(APlusBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AMinusBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AMinusBinop} node
	 * @param question the provided question
	 */
	public void caseAMinusBinop(AMinusBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ATimesBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link ATimesBinop} node
	 * @param question the provided question
	 */
	public void caseATimesBinop(ATimesBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ADivideBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link ADivideBinop} node
	 * @param question the provided question
	 */
	public void caseADivideBinop(ADivideBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AEqBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AEqBinop} node
	 * @param question the provided question
	 */
	public void caseAEqBinop(AEqBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AGtBinop} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AGtBinop} node
	 * @param question the provided question
	 */
	public void caseAGtBinop(AGtBinop node, Q question) {
		defaultPBinop(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ABinopExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ABinopExp} node
	 * @param question the provided question
	 */
	public void caseABinopExp(ABinopExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link APointerExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link APointerExp} node
	 * @param question the provided question
	 */
	public void caseAPointerExp(APointerExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ADerefExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ADerefExp} node
	 * @param question the provided question
	 */
	public void caseADerefExp(ADerefExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AIntConstExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AIntConstExp} node
	 * @param question the provided question
	 */
	public void caseAIntConstExp(AIntConstExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AFunctionCallExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AFunctionCallExp} node
	 * @param question the provided question
	 */
	public void caseAFunctionCallExp(AFunctionCallExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AFunctionPtrCallExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AFunctionPtrCallExp} node
	 * @param question the provided question
	 */
	public void caseAFunctionPtrCallExp(AFunctionPtrCallExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AVarExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AVarExp} node
	 * @param question the provided question
	 */
	public void caseAVarExp(AVarExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AInputExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AInputExp} node
	 * @param question the provided question
	 */
	public void caseAInputExp(AInputExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AMallocExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AMallocExp} node
	 * @param question the provided question
	 */
	public void caseAMallocExp(AMallocExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ANullExp} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ANullExp} node
	 * @param question the provided question
	 */
	public void caseANullExp(ANullExp node, Q question) {
		defaultPExp(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AIfThenStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AIfThenStm} node
	 * @param question the provided question
	 */
	public void caseAIfThenStm(AIfThenStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AIfThenElseStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AIfThenElseStm} node
	 * @param question the provided question
	 */
	public void caseAIfThenElseStm(AIfThenElseStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AWhileStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AWhileStm} node
	 * @param question the provided question
	 */
	public void caseAWhileStm(AWhileStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ADeclStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link ADeclStm} node
	 * @param question the provided question
	 */
	public void caseADeclStm(ADeclStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AReturnStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AReturnStm} node
	 * @param question the provided question
	 */
	public void caseAReturnStm(AReturnStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AOutputStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AOutputStm} node
	 * @param question the provided question
	 */
	public void caseAOutputStm(AOutputStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AAssignVarStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AAssignVarStm} node
	 * @param question the provided question
	 */
	public void caseAAssignVarStm(AAssignVarStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link AAssignPtrStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AAssignPtrStm} node
	 * @param question the provided question
	 */
	public void caseAAssignPtrStm(AAssignPtrStm node, Q question) {
		defaultPStm(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link ABlockStm} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link ABlockStm} node
	 * @param question the provided question
	 */
	public void caseABlockStm(ABlockStm node, Q question) {
		defaultPStm(node, question);
	}
	

	/**
	 * Action for applying a question to a {@link TWhiteSpace} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TWhiteSpace} node
	 * @param question the provided question
	 */
	public void caseTWhiteSpace(TWhiteSpace node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TCommentSingle} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TCommentSingle} node
	 * @param question the provided question
	 */
	public void caseTCommentSingle(TCommentSingle node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TCommentBlock} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TCommentBlock} node
	 * @param question the provided question
	 */
	public void caseTCommentBlock(TCommentBlock node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TVar} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TVar} node
	 * @param question the provided question
	 */
	public void caseTVar(TVar node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TInput} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TInput} node
	 * @param question the provided question
	 */
	public void caseTInput(TInput node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TOutput} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TOutput} node
	 * @param question the provided question
	 */
	public void caseTOutput(TOutput node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TIf} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIf} node
	 * @param question the provided question
	 */
	public void caseTIf(TIf node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TElse} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TElse} node
	 * @param question the provided question
	 */
	public void caseTElse(TElse node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TWhile} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TWhile} node
	 * @param question the provided question
	 */
	public void caseTWhile(TWhile node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TReturn} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TReturn} node
	 * @param question the provided question
	 */
	public void caseTReturn(TReturn node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TMalloc} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TMalloc} node
	 * @param question the provided question
	 */
	public void caseTMalloc(TMalloc node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TNull} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TNull} node
	 * @param question the provided question
	 */
	public void caseTNull(TNull node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TLPar} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TLPar} node
	 * @param question the provided question
	 */
	public void caseTLPar(TLPar node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TRPar} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TRPar} node
	 * @param question the provided question
	 */
	public void caseTRPar(TRPar node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TLBrace} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TLBrace} node
	 * @param question the provided question
	 */
	public void caseTLBrace(TLBrace node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TRBrace} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TRBrace} node
	 * @param question the provided question
	 */
	public void caseTRBrace(TRBrace node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TSemicolon} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TSemicolon} node
	 * @param question the provided question
	 */
	public void caseTSemicolon(TSemicolon node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TComma} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TComma} node
	 * @param question the provided question
	 */
	public void caseTComma(TComma node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TAssign} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TAssign} node
	 * @param question the provided question
	 */
	public void caseTAssign(TAssign node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TPlus} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TPlus} node
	 * @param question the provided question
	 */
	public void caseTPlus(TPlus node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TMinus} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TMinus} node
	 * @param question the provided question
	 */
	public void caseTMinus(TMinus node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TStar} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TStar} node
	 * @param question the provided question
	 */
	public void caseTStar(TStar node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TDiv} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TDiv} node
	 * @param question the provided question
	 */
	public void caseTDiv(TDiv node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TAnd} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TAnd} node
	 * @param question the provided question
	 */
	public void caseTAnd(TAnd node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TGt} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TGt} node
	 * @param question the provided question
	 */
	public void caseTGt(TGt node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TEq} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TEq} node
	 * @param question the provided question
	 */
	public void caseTEq(TEq node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TIntegerConst} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIntegerConst} node
	 * @param question the provided question
	 */
	public void caseTIntegerConst(TIntegerConst node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link TIdentifier} node. 
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIdentifier} node
	 * @param question the provided question
	 */
	public void caseTIdentifier(TIdentifier node, Q question) {
		defaultToken(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PProgram} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PProgram} node
	 * @param question the provided question
	 */
	public void defaultPProgram(PProgram node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PDecl} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PDecl} node
	 * @param question the provided question
	 */
	public void defaultPDecl(PDecl node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PBody} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBody} node
	 * @param question the provided question
	 */
	public void defaultPBody(PBody node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PBlock} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBlock} node
	 * @param question the provided question
	 */
	public void defaultPBlock(PBlock node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PBinop} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBinop} node
	 * @param question the provided question
	 */
	public void defaultPBinop(PBinop node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PExp} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PExp} node
	 * @param question the provided question
	 */
	public void defaultPExp(PExp node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link PStm} node.
	 * The call is deferred to {@link QuestionAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PStm} node
	 * @param question the provided question
	 */
	public void defaultPStm(PStm node, Q question) {
		defaultNode(node, question);
	}
	
	/**
	 * Action for applying a question to a {@link EOF} node.
	 * The call is deferred to {@link QuestionAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link EOF} node
	 * @param question the provided question
	 */
    public void caseEOF(EOF node, Q question) {
		defaultToken(node, question);
	}

}
