
package tip.analysis;

import tip.node.*;

/**
 * {@code QuestionAnswerAdapter} implements the default behaviour of the
 * {@link QuestionAnswer} interface. All calls return {@code null} by default.
 *
 * @param <Q> the type of the questions
 * @param <A> the type of the answers
 *
 * @author Johnni Winther, jw@brics.dk
 */
public class QuestionAnswerAdapter<Q,A> implements QuestionAnswer<Q,A> {

	/**
	 * Returns the default answer for a {@link Node}.
	 * @param node the calling {@link Node}
	 * @param question the question
	 * @return the default answer for a {@link Node}
	 */
	public A defaultNode(Node node, Q question) {
		return null;
	}
	
	/**
	 * Returns the default answer for a {@link Token}. The call is deferred to
	 * {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param token the calling {@link Token}
	 * @param question the question
	 * @return the default answer for a {@link Token}
	 */
	public A defaultToken(Token token, Q question) {
		return defaultNode(token, question);
	}
	
	/**
	 * Returns the default answer for a {@link Start} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link Start} node
	 * @param question the question
	 * @return the default answer for a {@link Start} node
	 */
    public A caseStart(Start node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AProgram} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPProgram(PProgram,Object)}.
	 * @param node the calling {@link AProgram} node
	 * @param question the question
	 * @return the default answer for a {@link AProgram} node
	 */
	public A caseAProgram(AProgram node, Q question) {
		return defaultPProgram(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AFuncDecl} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPDecl(PDecl,Object)}.
	 * @param node the calling {@link AFuncDecl} node
	 * @param question the question
	 * @return the default answer for a {@link AFuncDecl} node
	 */
	public A caseAFuncDecl(AFuncDecl node, Q question) {
		return defaultPDecl(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ALocalDecl} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPDecl(PDecl,Object)}.
	 * @param node the calling {@link ALocalDecl} node
	 * @param question the question
	 * @return the default answer for a {@link ALocalDecl} node
	 */
	public A caseALocalDecl(ALocalDecl node, Q question) {
		return defaultPDecl(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ABody} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBody(PBody,Object)}.
	 * @param node the calling {@link ABody} node
	 * @param question the question
	 * @return the default answer for a {@link ABody} node
	 */
	public A caseABody(ABody node, Q question) {
		return defaultPBody(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ABlock} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBlock(PBlock,Object)}.
	 * @param node the calling {@link ABlock} node
	 * @param question the question
	 * @return the default answer for a {@link ABlock} node
	 */
	public A caseABlock(ABlock node, Q question) {
		return defaultPBlock(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link APlusBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link APlusBinop} node
	 * @param question the question
	 * @return the default answer for a {@link APlusBinop} node
	 */
	public A caseAPlusBinop(APlusBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AMinusBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AMinusBinop} node
	 * @param question the question
	 * @return the default answer for a {@link AMinusBinop} node
	 */
	public A caseAMinusBinop(AMinusBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ATimesBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link ATimesBinop} node
	 * @param question the question
	 * @return the default answer for a {@link ATimesBinop} node
	 */
	public A caseATimesBinop(ATimesBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ADivideBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link ADivideBinop} node
	 * @param question the question
	 * @return the default answer for a {@link ADivideBinop} node
	 */
	public A caseADivideBinop(ADivideBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AEqBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AEqBinop} node
	 * @param question the question
	 * @return the default answer for a {@link AEqBinop} node
	 */
	public A caseAEqBinop(AEqBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AGtBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPBinop(PBinop,Object)}.
	 * @param node the calling {@link AGtBinop} node
	 * @param question the question
	 * @return the default answer for a {@link AGtBinop} node
	 */
	public A caseAGtBinop(AGtBinop node, Q question) {
		return defaultPBinop(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ABinopExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ABinopExp} node
	 * @param question the question
	 * @return the default answer for a {@link ABinopExp} node
	 */
	public A caseABinopExp(ABinopExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link APointerExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link APointerExp} node
	 * @param question the question
	 * @return the default answer for a {@link APointerExp} node
	 */
	public A caseAPointerExp(APointerExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ADerefExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ADerefExp} node
	 * @param question the question
	 * @return the default answer for a {@link ADerefExp} node
	 */
	public A caseADerefExp(ADerefExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AIntConstExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AIntConstExp} node
	 * @param question the question
	 * @return the default answer for a {@link AIntConstExp} node
	 */
	public A caseAIntConstExp(AIntConstExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AFunctionCallExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AFunctionCallExp} node
	 * @param question the question
	 * @return the default answer for a {@link AFunctionCallExp} node
	 */
	public A caseAFunctionCallExp(AFunctionCallExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AFunctionPtrCallExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AFunctionPtrCallExp} node
	 * @param question the question
	 * @return the default answer for a {@link AFunctionPtrCallExp} node
	 */
	public A caseAFunctionPtrCallExp(AFunctionPtrCallExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AVarExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AVarExp} node
	 * @param question the question
	 * @return the default answer for a {@link AVarExp} node
	 */
	public A caseAVarExp(AVarExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AInputExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AInputExp} node
	 * @param question the question
	 * @return the default answer for a {@link AInputExp} node
	 */
	public A caseAInputExp(AInputExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AMallocExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link AMallocExp} node
	 * @param question the question
	 * @return the default answer for a {@link AMallocExp} node
	 */
	public A caseAMallocExp(AMallocExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ANullExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPExp(PExp,Object)}.
	 * @param node the calling {@link ANullExp} node
	 * @param question the question
	 * @return the default answer for a {@link ANullExp} node
	 */
	public A caseANullExp(ANullExp node, Q question) {
		return defaultPExp(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AIfThenStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AIfThenStm} node
	 * @param question the question
	 * @return the default answer for a {@link AIfThenStm} node
	 */
	public A caseAIfThenStm(AIfThenStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AIfThenElseStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AIfThenElseStm} node
	 * @param question the question
	 * @return the default answer for a {@link AIfThenElseStm} node
	 */
	public A caseAIfThenElseStm(AIfThenElseStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AWhileStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AWhileStm} node
	 * @param question the question
	 * @return the default answer for a {@link AWhileStm} node
	 */
	public A caseAWhileStm(AWhileStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ADeclStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link ADeclStm} node
	 * @param question the question
	 * @return the default answer for a {@link ADeclStm} node
	 */
	public A caseADeclStm(ADeclStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AReturnStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AReturnStm} node
	 * @param question the question
	 * @return the default answer for a {@link AReturnStm} node
	 */
	public A caseAReturnStm(AReturnStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AOutputStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AOutputStm} node
	 * @param question the question
	 * @return the default answer for a {@link AOutputStm} node
	 */
	public A caseAOutputStm(AOutputStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AAssignVarStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AAssignVarStm} node
	 * @param question the question
	 * @return the default answer for a {@link AAssignVarStm} node
	 */
	public A caseAAssignVarStm(AAssignVarStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link AAssignPtrStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link AAssignPtrStm} node
	 * @param question the question
	 * @return the default answer for a {@link AAssignPtrStm} node
	 */
	public A caseAAssignPtrStm(AAssignPtrStm node, Q question) {
		return defaultPStm(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link ABlockStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultPStm(PStm,Object)}.
	 * @param node the calling {@link ABlockStm} node
	 * @param question the question
	 * @return the default answer for a {@link ABlockStm} node
	 */
	public A caseABlockStm(ABlockStm node, Q question) {
		return defaultPStm(node, question);
	}
	

	/**
	 * Returns the default answer for a {@link TWhiteSpace} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TWhiteSpace} node
	 * @param question the question
	 * @return the default answer for a {@link TWhiteSpace} node
	 */
	public A caseTWhiteSpace(TWhiteSpace node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TCommentSingle} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TCommentSingle} node
	 * @param question the question
	 * @return the default answer for a {@link TCommentSingle} node
	 */
	public A caseTCommentSingle(TCommentSingle node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TCommentBlock} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TCommentBlock} node
	 * @param question the question
	 * @return the default answer for a {@link TCommentBlock} node
	 */
	public A caseTCommentBlock(TCommentBlock node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TVar} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TVar} node
	 * @param question the question
	 * @return the default answer for a {@link TVar} node
	 */
	public A caseTVar(TVar node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TInput} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TInput} node
	 * @param question the question
	 * @return the default answer for a {@link TInput} node
	 */
	public A caseTInput(TInput node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TOutput} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TOutput} node
	 * @param question the question
	 * @return the default answer for a {@link TOutput} node
	 */
	public A caseTOutput(TOutput node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TIf} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIf} node
	 * @param question the question
	 * @return the default answer for a {@link TIf} node
	 */
	public A caseTIf(TIf node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TElse} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TElse} node
	 * @param question the question
	 * @return the default answer for a {@link TElse} node
	 */
	public A caseTElse(TElse node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TWhile} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TWhile} node
	 * @param question the question
	 * @return the default answer for a {@link TWhile} node
	 */
	public A caseTWhile(TWhile node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TReturn} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TReturn} node
	 * @param question the question
	 * @return the default answer for a {@link TReturn} node
	 */
	public A caseTReturn(TReturn node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TMalloc} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TMalloc} node
	 * @param question the question
	 * @return the default answer for a {@link TMalloc} node
	 */
	public A caseTMalloc(TMalloc node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TNull} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TNull} node
	 * @param question the question
	 * @return the default answer for a {@link TNull} node
	 */
	public A caseTNull(TNull node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TLPar} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TLPar} node
	 * @param question the question
	 * @return the default answer for a {@link TLPar} node
	 */
	public A caseTLPar(TLPar node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TRPar} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TRPar} node
	 * @param question the question
	 * @return the default answer for a {@link TRPar} node
	 */
	public A caseTRPar(TRPar node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TLBrace} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TLBrace} node
	 * @param question the question
	 * @return the default answer for a {@link TLBrace} node
	 */
	public A caseTLBrace(TLBrace node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TRBrace} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TRBrace} node
	 * @param question the question
	 * @return the default answer for a {@link TRBrace} node
	 */
	public A caseTRBrace(TRBrace node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TSemicolon} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TSemicolon} node
	 * @param question the question
	 * @return the default answer for a {@link TSemicolon} node
	 */
	public A caseTSemicolon(TSemicolon node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TComma} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TComma} node
	 * @param question the question
	 * @return the default answer for a {@link TComma} node
	 */
	public A caseTComma(TComma node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TAssign} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TAssign} node
	 * @param question the question
	 * @return the default answer for a {@link TAssign} node
	 */
	public A caseTAssign(TAssign node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TPlus} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TPlus} node
	 * @param question the question
	 * @return the default answer for a {@link TPlus} node
	 */
	public A caseTPlus(TPlus node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TMinus} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TMinus} node
	 * @param question the question
	 * @return the default answer for a {@link TMinus} node
	 */
	public A caseTMinus(TMinus node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TStar} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TStar} node
	 * @param question the question
	 * @return the default answer for a {@link TStar} node
	 */
	public A caseTStar(TStar node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TDiv} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TDiv} node
	 * @param question the question
	 * @return the default answer for a {@link TDiv} node
	 */
	public A caseTDiv(TDiv node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TAnd} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TAnd} node
	 * @param question the question
	 * @return the default answer for a {@link TAnd} node
	 */
	public A caseTAnd(TAnd node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TGt} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TGt} node
	 * @param question the question
	 * @return the default answer for a {@link TGt} node
	 */
	public A caseTGt(TGt node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TEq} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TEq} node
	 * @param question the question
	 * @return the default answer for a {@link TEq} node
	 */
	public A caseTEq(TEq node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TIntegerConst} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIntegerConst} node
	 * @param question the question
	 * @return the default answer for a {@link TIntegerConst} node
	 */
	public A caseTIntegerConst(TIntegerConst node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link TIdentifier} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link TIdentifier} node
	 * @param question the question
	 * @return the default answer for a {@link TIdentifier} node
	 */
	public A caseTIdentifier(TIdentifier node, Q question) {
		return defaultToken(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PProgram} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PProgram} node
	 * @param question the question
	 * @return the default answer for a {@link PProgram} node
	 */
	public A defaultPProgram(PProgram node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PDecl} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PDecl} node
	 * @param question the question
	 * @return the default answer for a {@link PDecl} node
	 */
	public A defaultPDecl(PDecl node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PBody} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBody} node
	 * @param question the question
	 * @return the default answer for a {@link PBody} node
	 */
	public A defaultPBody(PBody node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PBlock} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBlock} node
	 * @param question the question
	 * @return the default answer for a {@link PBlock} node
	 */
	public A defaultPBlock(PBlock node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PBinop} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PBinop} node
	 * @param question the question
	 * @return the default answer for a {@link PBinop} node
	 */
	public A defaultPBinop(PBinop node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PExp} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PExp} node
	 * @param question the question
	 * @return the default answer for a {@link PExp} node
	 */
	public A defaultPExp(PExp node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link PStm} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultNode(Node,Object)}.
	 * @param node the calling {@link PStm} node
	 * @param question the question
	 * @return the default answer for a {@link PStm} node
	 */
	public A defaultPStm(PStm node, Q question) {
		return defaultNode(node, question);
	}
	
	/**
	 * Returns the default answer for a {@link EOF} node. The call is deferred
	 * to {@link QuestionAnswerAdapter#defaultToken(Token,Object)}.
	 * @param node the calling {@link EOF} node
	 * @param question the question
	 * @return the default answer for a {@link EOF} node
	 */
    public A caseEOF(EOF node, Q question) {
		return defaultToken(node, question);
	}

}
