package tip.analysis;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import tip.node.*;

public class TreeDisplay extends DepthFirstAdapter {
	private Stack<DefaultMutableTreeNode> parents = new Stack<DefaultMutableTreeNode>();
	private Map<Node,Integer> nodeIndexMap = new HashMap<Node,Integer>();
	protected String displayName = "TreeDisplay";
	protected boolean expandAll = true;
	protected boolean waitOnDisplay = true;
	private boolean waiting = false;
	
	protected void displayTree(DefaultMutableTreeNode root) {
		JFrame frame = new JFrame(displayName);
		JTree tree = new JTree(root);
		JScrollPane pane = new JScrollPane (tree);
		
		//expandAll (tree);
		
		if (waitOnDisplay) {
			waiting = true;
			frame.addWindowListener(new WindowAdapter () {
				public void windowClosing(WindowEvent e) {
					synchronized (TreeDisplay.this) {
						waiting = false;
						TreeDisplay.this.notify();
					}
				}
			});
		}
		frame.setSize(600, 800);
		frame.getContentPane().add(pane);
		frame.setVisible(true);
		try {
			while (waiting) {
				synchronized (this) {
					wait();
				}
			}
		} catch(InterruptedException e) {
		}
	}
	@Override
	public void defaultIn(Node node) {
		int nodeIndex = nodeIndexMap.size();
		if (nodeIndexMap.containsKey(node)) {
			nodeIndex = nodeIndexMap.get(node);
		} else {
			nodeIndexMap.put(node, nodeIndex);
		}
		parents.push(new DefaultMutableTreeNode(node.getClass().getSimpleName() + " : " + nodeIndex));
	}
	@Override
	public void defaultOut(Node node) {
		if (!parents.isEmpty()) {
			DefaultMutableTreeNode treeNode = parents.pop();
			if (!parents.isEmpty()) {
				parents.peek().add(treeNode);
			} else {
				displayTree(treeNode);
			}
		}
	}
	@Override
	public void defaultToken(Token node) {
		int nodeIndex = nodeIndexMap.size();
		if (nodeIndexMap.containsKey(node)) {
			nodeIndex = nodeIndexMap.get(node);
		} else {
			nodeIndexMap.put(node, nodeIndex);
		}
		DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode(node.getClass().getSimpleName() + "(\"" + node.getText() + "\") : " + nodeIndex);
		if (!parents.isEmpty()) {
			parents.peek().add(treeNode);
		} else {
			displayTree(treeNode);
		}
	}
}
