/* This file was generated by SableCC (http://www.sablecc.org/). */
 
package tip.cfg;

import java.util.*;

import tip.node.*;

/**
 * {@code AProgramPoint} implements the basic functionality of the generated
 * program points.
 *
 * @author Johnni Winther, jw@brics.dk
 */
public abstract class AProgramPoint implements IProgramPoint {
	// the node associated with the program point
	protected Node _node_;
	// the set of predecessor program points
	protected Set<IProgramPoint> _predecessor_set_ = new LinkedHashSet<IProgramPoint>();
	// the set of successor program points
	protected Set<IProgramPoint> _successor_set_ = new LinkedHashSet<IProgramPoint>();
	
	/**
	 * Creates a new program point associated with {@code _node_}.
	 * @param _node_ the {@link Node} associated with this program point
	 */ 
	public AProgramPoint(Node _node_) {
		this._node_ = _node_;
	}
	
	/**
	 * Adds {@code predecessor} to the set of predecessors of this program point.
	 * The program point as implicitly added to the successor set of the the
	 * predecessor. A program point can only be predecessor of a program point once.
	 * @param predecessor the program point added to the predecessor set
	 */
	public void addPredecessor(IProgramPoint predecessor) {
		if (predecessor == null) return;
		if (this._predecessor_set_.add(predecessor)) {
			predecessor.addSuccessor(this);
		}
	}
	
	/**
	 * Removes {@code predecessor} from the set of predecessors of this program point.
	 * The program point as implicitly removed from the successor set of the the
	 * predecessor.
	 * @param predecessor the program point removed from the predecessor set
	 */
	public void removePredecessor(IProgramPoint predecessor) {
		if (predecessor == null) return;
		if (this._predecessor_set_.remove(predecessor)) {
			predecessor.removeSuccessor(this);
		}
	}
	
	/**
	 * Adds {@code successor} to the set of successors of this program point.
	 * The program point as implicitly added to the predecessor set of the the
	 * successor. A program point can only be successor of a program point once.
	 * @param successor the program point added to the successor set
	 */
	public void addSuccessor(IProgramPoint successor) {
		if (successor == null) return;
		if (this._successor_set_.add(successor)) {
			successor.addPredecessor(this);
		}
	}
	
	/**
	 * Removes {@code successor} to the set of successors of this program point.
	 * The program point as implicitly removes from the predecessor set of the the
	 * successor.
	 * @param successor the program point removed from the successor set
	 */
	public void removeSuccessor(IProgramPoint successor) {
		if (successor == null) return;
		if (this._successor_set_.remove(successor)) {
			successor.removePredecessor(this);
		}
	}
	
	/**
	 * Returns the set of predecessors to this program point.
	 * @return the set of predecessors to this program point
	 */
	public Set<IProgramPoint> getPredecessors() {
		return this._predecessor_set_;
	}
	
	/**
	 * Returns the set of successors to this program point.
	 * @return the set of successors to this program point
	 */
	public Set<IProgramPoint> getSuccessors() {
		return this._successor_set_;
	}
	
	/**
	 * Returns the set of program points reachable from this
	 * point following at most {@code distance} successors
	 * or predecessors.
	 * @param distance the maximum distance of points returned
	 * @return the set of program points in the {@code distance}-vicinity of this point
	 */
	public Set<IProgramPoint> getVicinity(int distance) {
		Set<IProgramPoint> vicinity = new HashSet<IProgramPoint>();
		Set<IProgramPoint> this_level = new HashSet<IProgramPoint>();
		this_level.add(this);
		while (distance >= 0) {
			distance--;
			vicinity.addAll(this_level);
			Set<IProgramPoint> next_level = new HashSet<IProgramPoint>();
			for (IProgramPoint point : this_level) {
				for (IProgramPoint successor : point.getSuccessors()) {
					if (!vicinity.contains(successor)) {
						next_level.add(successor);
					}
				}
				for (IProgramPoint predecessor : point.getPredecessors()) {
					if (!vicinity.contains(predecessor)) {
						next_level.add(predecessor);
					}
				}
			}
			if (next_level.isEmpty()) break;
			this_level = next_level;
		}
		return vicinity;
	}
	
	/**
	 * Returns the {@link Node} in the AST associated with this program point.
	 * @return the {@link Node} in the AST associated with this program point
	 */
	public Node getNode() {
		return this._node_;
	}
	
	/**
	 * Returns a clone of the program point with no predecessors or successors.
	 * @return a clone of the program point with no predecessors or successors
	 */
	public abstract AProgramPoint clone();
}
