/* This file was generated by SableCC (http://www.sablecc.org/). */
 
package tip.cfg;


import tip.node.*;
import tip.analysis.*;
import java.util.*;

/**
 * {@code CCFG} is the standard implementation of {@link ICFG}. It is used by
 * {@link CFGBuilder}.
 *
 * @author Johnni Winther, jw@brics.dk
 */
public class CCFG implements ICFG {
	// the name of the control flow graph
	protected String _name_;
	
	protected Set<IProgramPoint> _program_points_ = new LinkedHashSet<IProgramPoint>();
	// the entry point of the control flow graph
	protected IProgramPoint _entry_;
	// the exit point of the control flow graph
	protected IProgramPoint _exit_;
	
	/**
	 * Creates a new empty control flow graph with the name {@code name}.
	 * @param name the name of the control flow graph
	 */
	public CCFG(String name) {
		this._name_ = name;
	}
	
	/**
	 * Creates a new linear control flow graph from the list of program
	 * points. The first point is set to be the entry point, the last
	 * to be the exit point, and predecessor/successors relations are
	 * set between the points to form a linear control flow graph.
	 * @param name the name of the control flow graph
	 * @param pointList the list of program points from which the control flow graph is constructed
	 */
	public CCFG(String name, List<IProgramPoint> pointList) {
		this._name_ = name;
		this._program_points_.addAll(pointList);
		this._entry_ = pointList.get(0);
		this._exit_ = pointList.get(pointList.size()-1);
		for (int i = 1 ; i < pointList.size() ; i++) {
			pointList.get(i).addPredecessor(pointList.get(i-1));
		}
	}
	
	/**
	 * Creates a new control flow graph from the set of control flow graphs.
	 * A new entry and exit point (with no associated {@link Node}) are made,
	 * all successors of the entry points are set as successors to this new
	 * entry point and all predecessors of the exit points are set as 
	 * predecessors of the new exit point.
	 * @param name the name of the control flow graph
	 * @param cfgSet the set of control flow graphs from which the control flow graph is constructed
	 */
	public CCFG(String name, Set<ICFG> cfgSet) {
		this._name_ = name;
		this._entry_ = new EntryPoint();
		this._exit_ = new ExitPoint();
		if (cfgSet == null || cfgSet.isEmpty()) {
		} else {
			for (ICFG cfg : cfgSet) {
				this._program_points_.addAll(cfg.getProgramPoints());
				for (IProgramPoint successor : cfg.getEntry().getSuccessors()) {
					successor.addPredecessor(this._entry_);
				}
				for (IProgramPoint predecessor : cfg.getExit().getPredecessors()) {
					predecessor.addSuccessor(this._exit_);
				}
				removePoint(cfg.getEntry());
				removePoint(cfg.getExit());
			}
		}
	}
	
	/**
	 * Returns the name of the control flow graph
	 * @return the name of the control flow graph
	 */
	public String getName() {
		return this._name_;
	}
	
	/**
	 * Adds the {@link IProgramPoint} {@code point} to the set
	 * of program points associated with this {@link ICFG}.
	 * Predecessors and successors are <em>not</em> updated.
	 * @param point the {@link IProgramPoint} to be added to this {@link ICFG}
	 */
	public void add(IProgramPoint point) {
		if (point == null) return;
		_program_points_.add(point);
	}
	
	/**
	 * Removes the {@link IProgramPoint} {@code point} from the set
	 * of program points associated with this {@link ICFG}.
	 * Predecessors and successors are <em>not</em> updated.
	 * @param point the {@link IProgramPoint} to be removed from this {@link ICFG}
	 */
	public void remove(IProgramPoint point) {
		if (point == null) return;
		_program_points_.remove(point);
	}

	/**
	 * Returns the collection of program points in the control flow graph.
	 * @return the collection of program points in the control flow graph
	 */
	public Collection<IProgramPoint> getProgramPoints() {
		return this._program_points_;
	}
	
	/**
	 * Returns {@code true} if this {@link ICFG} contains the {@link IProgramPoint} {@code point}.
	 * @param point the {@link IProgramPoint} in question
	 * @return {@code true} if this {@link ICFG} contains the {@link IProgramPoint} {@code point}
	 */
	public boolean containsProgramPoint(IProgramPoint point) {
		return this._program_points_.contains(point);
	}
	
	/**
	 * Replaces {@code point} with {@code cfg} in this control flow graph.
	 * That is, {@code point} is removed from this control flow graph, 
	 * all predecessors of {@code point} are set to have all successors
	 * of the entry point in {@code cfg} as their successor instead of 
	 * {@code point} and all successors of {@code point} are set to have
	 * all predecessors of the exit point in {@code cfg} as their predecessor
	 * instead of {@code point}. All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param point the replaced point
	 * @param cfg the replacing control flow graph
	 */
	public void replaceByCFG(IProgramPoint point, ICFG cfg) {
		this._program_points_.addAll(cfg.getProgramPoints());
		boolean entryToExitLink = false;
		Set<IProgramPoint> pointPredecessorSet = new HashSet<IProgramPoint>(point.getPredecessors());
		Set<IProgramPoint> pointSuccessorSet = new HashSet<IProgramPoint>(point.getSuccessors());
		for (IProgramPoint pointPredecessor : pointPredecessorSet) {
			point.removePredecessor(pointPredecessor);
			for (IProgramPoint entrySuccessor : new HashSet<IProgramPoint>(cfg.getEntry().getSuccessors())) {
				if (entrySuccessor.equals(cfg.getExit())) {
					entryToExitLink = true;
				} else {
					pointPredecessor.addSuccessor(entrySuccessor);
				}
			}
		}
		for (IProgramPoint pointSucccessor : pointSuccessorSet) {
			point.removeSuccessor(pointSucccessor);
			for (IProgramPoint exitPredecessor : new HashSet<IProgramPoint>(cfg.getExit().getPredecessors())) {
				if (exitPredecessor.equals(cfg.getEntry())) {
					entryToExitLink = true;
				} else {
					pointSucccessor.addPredecessor(exitPredecessor);
				}
			}
		}
		if (entryToExitLink) {
			for (IProgramPoint pointPredecessor : pointPredecessorSet) {
				for (IProgramPoint pointSuccessor : pointSuccessorSet) {
					pointPredecessor.addSuccessor(pointSuccessor);
				}
			}
		}
		remove(cfg.getEntry());
		remove(cfg.getExit());
		remove(point);
	}
	
	/**
	 * Prefixes {@code cfg} in this control flow graph. That is, 
	 * all successors of the entry point in {@code cfg} are set to be
	 * successors of the entry point in this control flow graph, and
	 * all successors of the entry point in this control flow graph
	 * are set as successors of the exit point in {@code cfg}.
	 * All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param cfg the prefixed cfg
	 */
	public void prefixCFG(ICFG cfg) {
		this._program_points_.addAll(cfg.getProgramPoints());
		Set<IProgramPoint> exitPredecessorSet = new HashSet<IProgramPoint>(cfg.getExit().getPredecessors());
		Set<IProgramPoint> entrySuccessorSet = new HashSet<IProgramPoint>(getEntry().getSuccessors());
		for (IProgramPoint pointSuccessor : entrySuccessorSet) {
			this._entry_.removeSuccessor(pointSuccessor);
			for (IProgramPoint pointPredecessor : exitPredecessorSet) {
					pointPredecessor.addSuccessor(pointSuccessor);
			}
		}
		remove(cfg.getExit());
		remove(getEntry());
		setEntry(cfg.getEntry());
	}
	
	/**
	 * Suffixes {@code cfg} in this control flow graph. That is, 
	 * all predecessors of the exit point in {@code cfg} are set to be
	 * predecessors of the exit point in this control flow graph, and
	 * all predecessors of the exit point in this control flow graph
	 * are set as predecessors of the entry point in {@code cfg}.
	 * All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param cfg the suffixed cfg
	 */
	public void suffixCFG(ICFG cfg) {
		this._program_points_.addAll(cfg.getProgramPoints());
		Set<IProgramPoint> exitPredecessorSet = new HashSet<IProgramPoint>(getExit().getPredecessors());
		Set<IProgramPoint> entrySuccessorSet = new HashSet<IProgramPoint>(cfg.getEntry().getSuccessors());
		for (IProgramPoint pointPredecessor : exitPredecessorSet) {
			this._exit_.removePredecessor(pointPredecessor);
			for (IProgramPoint pointSuccessor : entrySuccessorSet) {
				pointPredecessor.addSuccessor(pointSuccessor);
			}
		}
		remove(getExit());
		remove(cfg.getEntry());
		setExit(cfg.getExit());
	}
	
	/**
	 * Replaces {@code oldPoint} with {@code newPoint} in this control flow graph.
	 * Successors and predecessors are transferred from {@code oldPoint} to 
	 * {@code newPoint}. {@code oldPoint} is removed from this control flow graph
	 * {@code newPoint} is added to this control flow graph.
	 * @param oldPoint the replaced point
	 * @param newPoint the replacing point
	 */
	public void replacePoint(IProgramPoint oldPoint, IProgramPoint newPoint) {
		for (IProgramPoint successor : new HashSet<IProgramPoint>(oldPoint.getSuccessors())) {
			oldPoint.removeSuccessor(successor);
			newPoint.addSuccessor(successor);
		}
		for (IProgramPoint predecessor : new HashSet<IProgramPoint>(oldPoint.getPredecessors())) {
			oldPoint.removePredecessor(predecessor);
			newPoint.addPredecessor(predecessor);
		}
		remove(oldPoint);
		add(newPoint);
	}
	
	/**
	 * Removes {@code point} from this control flow graph, updating the
	 * predecessors and successors of {@code point} to point to eachother.
	 * @param point the removed point
	 */
	public void removePoint(IProgramPoint point) {
		for (IProgramPoint successor : new HashSet<IProgramPoint>(point.getSuccessors())) {
			point.removeSuccessor(successor);
			for (IProgramPoint predecessor : new HashSet<IProgramPoint>(point.getPredecessors())) {
				point.removePredecessor(predecessor);
				successor.addPredecessor(predecessor);
			}
		}
		remove(point);
	}
	
	/**
	 * Inserts {@code newPoint} in the control flow graph after {@code point}.
	 * That is, all successors of {@code point} are set as successors of 
	 * {@code newPoint} and {@code newPoint} is set as the sole successor
	 * of {@code point}
	 * @param point the point in this control flow graph after which {@code newPoint} is inserted
	 * @param newPoint the inserted point
	 */
	public void insertAfter(IProgramPoint point, IProgramPoint newPoint) {
		for (IProgramPoint successor : new HashSet<IProgramPoint>(point.getSuccessors())) {
			point.removeSuccessor(successor);
			newPoint.addSuccessor(successor);
		}
		point.addSuccessor(newPoint);
		add(newPoint);
		if (point == this._exit_) {
			this._exit_ = newPoint;
		}
	}
	
	/**
	 * Inserts {@code newPoint} in the control flow graph before {@code point}.
	 * That is, all predecessors of {@code point} are set as predecessors of 
	 * {@code newPoint} and {@code newPoint} is set as the sole predecessor
	 * of {@code point}
	 * @param point the point in this control flow graph before which {@code newPoint} is inserted
	 * @param newPoint the inserted point
	 */
	public void insertBefore(IProgramPoint point, IProgramPoint newPoint) {
		for (IProgramPoint predecessor : new HashSet<IProgramPoint>(point.getPredecessors())) {
			point.removePredecessor(predecessor);
			newPoint.addPredecessor(predecessor);
		}
		point.addPredecessor(newPoint);
		add(newPoint);
		if (point == this._entry_) {
			this._entry_ = newPoint;
		}
	}
	
	/**
	 * Cleans the control flow graph by removing all predecessors and successors
	 * of the program points in the program point set of this control flow graph
	 * which are not in the program point set of this control flow graph. This
	 * may be necessary for analyses to work correctly after having added 
	 * points to the control flow graph from other control flow graphs, 
	 * as after {@link #replaceByCFG(IProgramPoint,ICFG)},
	 * {@link #prefixCFG(ICFG)}, {@link #suffixCFG(ICFG)}, 
	 * {@link #replacePoint(IProgramPoint,IProgramPoint)},
	 * {@link #insertAfter(IProgramPoint,IProgramPoint)}, and
	 * {@link #insertBefore(IProgramPoint,IProgramPoint)}.
	 */
	public void clean() {
		for (IProgramPoint point : _program_points_) {
			for (IProgramPoint successor : new HashSet<IProgramPoint>(point.getSuccessors())) {
				if (!containsProgramPoint(successor)) {
					point.removeSuccessor(successor);
				}
			}
			for (IProgramPoint predecessor : new HashSet<IProgramPoint>(point.getPredecessors())) {
				if (!containsProgramPoint(predecessor)) {
					point.removePredecessor(predecessor);
				}
			}
		}
	}
	
	/**
	 * Returns a collection of descendants to the nodes associated to the program
	 * points in this control flow graph. Only descendants that are subtypes of
	 * {@code classType} are returned.
	 * @param classType the supertype of returned descendants
	 * @return a collection of descendants to the associated nodes
	 */
	public <T extends Node> Collection<T> getNodeDescendants(Class<T> classType) {
		return getNodeDescendants(new SubTypeFilter<T>(classType));
	}
	
	/**
	 * Returns a collection of descendants to the nodes associated to the program
	 * points in this control flow graph. Only descendants that are accepted by
	 * the {@link NodeFilter} {@code filter} are returned.
	 * @param filter the {@link NodeFilter} used for filtering nodes
	 * @return a collection of descendants to the associated nodes
	 */
	public <T extends Node> Collection<T> getNodeDescendants(NodeFilter<T> filter) {
		Set<T> collection = new HashSet<T>();
		getNodeDescendants(collection, filter);
		return collection;
	}
	
	/**
	 * Adds descendants to the nodes associated to the program
	 * points in this control flow graph to {@code collection}.
	 * Only descendants that are accepted by the {@link NodeFilter} {@code filter}
	 * are returned.
	 * @param collection the collection to which the descendants are added
	 * @param filter the {@link NodeFilter} used for filtering nodes
	 */
	public <T extends Node> void getNodeDescendants(Collection<T> collection, NodeFilter<T> filter) {
		for (IProgramPoint point : getProgramPoints()) {
			Node node = point.getNode();
			if (node != null) {
				node.getDescendants(collection, filter);
			}
		}
	}
	
	/**
	 * Applies {@code analysis} to all nodes associated with program points
	 * in this control flow graph.
	 * @param analysis the {@link Analysis} applied to the associated nodes
	 */
	public void apply(Analysis analysis) {
		for (IProgramPoint point : getProgramPoints()) {
			Node node = point.getNode();
			if (node != null) {
				node.apply(analysis);
			}
		}
	}
	
	/**
	 * Applies the {@link Question} {@code caller} and {@code question} to all nodes
	 * associated with program points in this control flow graph.
	 * @param caller the {@link Question} applied to the associated nodes
	 * @param question the provided question
	 */
	public <Q> void apply(Question<Q> caller, Q question) {
		for (IProgramPoint point : getProgramPoints()) {
			Node node = point.getNode();
			if (node != null) {
				node.apply(caller, question);
			}
		}
	}
	
	/**
	 * Returns the entry point of the control flow graph.
	 * @return the entry point of the control flow graph
	 */
	public IProgramPoint getEntry() {
		return this._entry_;
	}
	
	/**
	 * Sets the entry point of the control flow graph.
	 * @param point the entry point of the control flow graph
	 * the graph
	 */
	public void setEntry(IProgramPoint point) {
		this._entry_ = point;
		add(point);
	}
	
	/**
	 * Returns the exit point of the control flow graph.
	 * @return the exit point of the control flow graph
	 */
	public IProgramPoint getExit() {
		return this._exit_;
	}
	
	/**
	 * Sets the exit point of the control flow graph.
	 * @param point the exit point of the control flow graph
	 * the graph
	 */
	public void setExit(IProgramPoint point) {
		this._exit_ = point;
		add(point);
	}
	
	/**
	 * Returns a deep clone of the ICFG with respect to program points,
	 * i.e. program points cloned but the associated AST nodes are not.
	 * @return a deep clone of the ICFG
	 */
	public CCFG clone() {
		CCFG cloneCFG = new CCFG(this._name_);
		Collection<IProgramPoint> programPoints = getProgramPoints();
		Map<IProgramPoint,IProgramPoint> old2newMap = new HashMap<IProgramPoint,IProgramPoint>();
		for (IProgramPoint point : programPoints) {
			IProgramPoint clonePoint = point.clone();
			old2newMap.put(point, clonePoint);
			cloneCFG.add(clonePoint);
		}
		for (IProgramPoint point : programPoints) {
			IProgramPoint clonePoint = old2newMap.get(point);
			for (IProgramPoint successor : point.getSuccessors()) {
				clonePoint.addSuccessor(old2newMap.get(successor));
			}
		}
		cloneCFG.setEntry(old2newMap.get(this._entry_));
		cloneCFG.setExit(old2newMap.get(this._exit_));
		return cloneCFG;
	}
	
	public String toString() {
		return this._name_ + ":" + hashCode();
	}
}
