/* This file was generated by SableCC (http://www.sablecc.org/). */
 
package tip.cfg;


import java.util.*;
import tip.node.*;
import tip.analysis.*;

/**
 * {@code CFGBuilder} constructs control flow graphs for an AST based on the
 * definitions in the SableCC grammar 'Control Flow Graph'-section.
 *
 * @author Johnni Winther, jw@brics.dk
 */
public class CFGBuilder extends DepthFirstAdapter {
	private static abstract class Vertex {
		public Graph parent;
		public Set<Vertex> edges = new HashSet<Vertex>();
		
		public Vertex(Graph parent) {
			this.parent = parent;
		}
		
		public abstract void getSuccessorPoints(Set<ProgramPointGraph> successorSet, Set<Vertex> vertexSet);
	}
	
	private static class EntryVertex extends Vertex {
		public EntryVertex(Graph parent) {
			super(parent);
		}
		
		public void getSuccessorPoints(Set<ProgramPointGraph> successorSet, Set<Vertex> vertexSet) {
			if (vertexSet.contains(this)) return;
			vertexSet.add(this);
			if (parent instanceof ProgramPointGraph) {
				successorSet.add((ProgramPointGraph)parent);
			} else {
				if (edges.size() != 0) {
					for (Vertex successor : edges) {
						successor.getSuccessorPoints(successorSet, vertexSet);
					}
				} else {
					parent.innerExit.getSuccessorPoints(successorSet, vertexSet);
				}
			}
		}
	}
	
	private static class ExitVertex extends Vertex {
		public ExitVertex(Graph parent) {
			super(parent);
		}
		
		public void getSuccessorPoints(Set<ProgramPointGraph> successorSet, Set<Vertex> vertexSet) {
			if (vertexSet.contains(this)) return;
			vertexSet.add(this);
			for (Vertex successor : edges) {
				successor.getSuccessorPoints(successorSet, vertexSet);
			}
		}
	}
	
	private static class Graph {
		public Node node;
		
		protected Vertex outerEntry = new EntryVertex(this);
		protected Vertex innerEntry = outerEntry;
		
		protected Vertex outerExit = new ExitVertex(this);
		protected Vertex innerExit = outerExit;
		
		protected Map<String,Vertex> entryMap = null;
		protected Map<String,Vertex> exitMap = null;
		
		public Graph(Node node) {
			this.node = node;
		}
		
		public Vertex getEntry() {
			return outerEntry;
		}
		
		public Vertex getEntry(String label) {
			if (entryMap == null) {
				entryMap = new HashMap<String,Vertex>();
				outerEntry = new EntryVertex(this);
			}
			Vertex vertex = entryMap.get(label);
			if (vertex == null) {
				vertex = new EntryVertex(this);
				entryMap.put(label, vertex);
				vertex.edges.add(innerEntry);
				outerEntry.edges.add(vertex);
			}
			return vertex;
		}
		
		public Vertex getInnerEntry() {
			return innerEntry; 
		}
		
		public Vertex getExit() {
			return outerExit;
		}
		
		public Vertex getExit(String label) {
			if (exitMap == null) {
				exitMap = new HashMap<String,Vertex>();
				outerExit = new ExitVertex(this);
				outerExit.edges.addAll(innerExit.edges);
				innerExit.edges.clear();
			}
			Vertex vertex = exitMap.get(label);
			if (vertex == null) {
				vertex = new ExitVertex(this);
				exitMap.put(label, vertex);
				vertex.edges.add(outerExit);
				innerExit.edges.add(vertex);
			}
			return vertex;
		}
		
		public Vertex getInnerExit() {
			return innerExit;
		}

		public String toString() {
			return node.toString();
		}
	}
	
	private static class ProgramPointGraph extends Graph {
		public AProgramPoint programPoint;
		
		public ProgramPointGraph(Node node, AProgramPoint programPoint) {
			super(node);
			this.programPoint = programPoint;
		}
		
		public void getSuccessorPoints(Set<ProgramPointGraph> programPointSet, Set<Graph> graphSet) {
			if (graphSet.contains(this)) return;
			graphSet.add(this);
			Set<Vertex> vertexSet = new HashSet<Vertex>();
			innerExit.getSuccessorPoints(programPointSet, vertexSet);
		}
	}
	
	private ProgramPointGraph currentEntryPoint;
	private ProgramPointGraph currentExitPoint;
	private HashMap<Node,Graph> graphMap;
	
	private CCFGCollection cfgCollection = new CCFGCollection();
	private NodeFilter filter = null;
	
	private CFGBuilder() {}
	
	private CFGBuilder(NodeFilter filter) {
		this.filter = filter;
	}
	
	private Graph filterGraph(Graph graph, Node node) {
		if (filter != null) {
			if (filter.accept(node)) {
				return graph;
			}
			return new Graph(node);
		}
		return graph;
	}
		
	private Graph getGraph(Node node) {
		if (graphMap == null) graphMap = new HashMap<Node,Graph>();
		Graph graph = graphMap.get(node);
		if (graph == null) {
			graph = new Graph(node);
			graphMap.put(node, graph);
		}
		return graph;
	}
	
	private void collectProgramPoints(CCFG cfg, ProgramPointGraph graph) {
		Set<ProgramPointGraph> pendingSet = new HashSet<ProgramPointGraph>();
		Set<ProgramPointGraph> handledSet = new HashSet<ProgramPointGraph>();
		pendingSet.add(graph);
		while (!pendingSet.isEmpty()) {
			graph = pendingSet.iterator().next();
			pendingSet.remove(graph);
			handledSet.add(graph);
			AProgramPoint programPoint = graph.programPoint;
			Set<ProgramPointGraph> programPointSet = new HashSet<ProgramPointGraph>();
			Set<Graph> graphSet = new HashSet<Graph>();
			graph.getSuccessorPoints(programPointSet, graphSet);
			for (ProgramPointGraph successor : programPointSet) {
				cfg.add(successor.programPoint);
				programPoint.addSuccessor(successor.programPoint);
				successor.programPoint.addPredecessor(programPoint);
				if (!handledSet.contains(successor)) {
					pendingSet.add(successor);
				}
			}
		}
	}
	
	public @Override void inABody(ABody node) {
		Graph nodeGraph = getGraph(node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getBlock());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getBlock());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAFunctionCallExp(AFunctionCallExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CFunctionCallExpPoint(node, "")), node);
		
		{

			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			ArrayList<Node> sub_nodes = new ArrayList<Node>(node.getArgs());

			for (Node sub_node : sub_nodes) {
				Graph elementGraph = getGraph(sub_node);
				Vertex elementVertex = elementGraph.getEntry();
				fromVertex.edges.add(elementVertex);
				fromVertex = elementGraph.getExit();
			}

			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}

		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAFuncDecl(AFuncDecl node) {
		Graph nodeGraph = getGraph(node);
		
		{

			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			ArrayList<Node> sub_nodes = new ArrayList<Node>(node.getFormals());

			for (Node sub_node : sub_nodes) {
				Graph elementGraph = getGraph(sub_node);
				Vertex elementVertex = elementGraph.getEntry();
				fromVertex.edges.add(elementVertex);
				fromVertex = elementGraph.getExit();
			}

			Graph toGraph = getGraph(node.getBody());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}

		{
		
			Graph fromGraph = getGraph(node.getBody());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
		currentEntryPoint = new ProgramPointGraph(node, new EntryPoint(node));
		currentExitPoint = new ProgramPointGraph(node, new ExitPoint(node));	
		{
			Vertex fromVertex = currentEntryPoint.getExit();

			Vertex toVertex = nodeGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		{
		
			Vertex fromVertex = nodeGraph.getExit();
			
			Vertex toVertex = currentExitPoint.getEntry();
			fromVertex.edges.add(toVertex);
		}

	}
			
	public @Override void outAFuncDecl(AFuncDecl node) {
		CCFG cfg = new CCFG(node.getName().getText());
		cfg.add(currentEntryPoint.programPoint);
		cfg.setEntry(currentEntryPoint.programPoint);
		collectProgramPoints(cfg, currentEntryPoint);
		cfg.add(currentExitPoint.programPoint);
		cfg.setExit(currentExitPoint.programPoint);
		cfgCollection.addCFG(cfg);
		graphMap = null;
	}
	
	public @Override void inAAssignPtrStm(AAssignPtrStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CAssignPtrStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getRight());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getRight());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getLeft());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getLeft());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAReturnStm(AReturnStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CReturnStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getExp());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getExp());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inADerefExp(ADerefExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CDerefExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getExp());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getExp());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAIfThenStm(AIfThenStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CIfThenStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getCondition());
			Vertex toVertex = toGraph.getEntry("true");
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getCondition());
			Vertex toVertex = toGraph.getEntry("false");
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("true");
			
			Graph toGraph = getGraph(node.getBody());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("false");
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getBody());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inANullExp(ANullExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CNullExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAVarExp(AVarExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CVarExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inABlock(ABlock node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CBlockPoint(node, "")), node);
		
		{

			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			ArrayList<Node> sub_nodes = new ArrayList<Node>(node.getStatements());

			for (Node sub_node : sub_nodes) {
				Graph elementGraph = getGraph(sub_node);
				Vertex elementVertex = elementGraph.getEntry();
				fromVertex.edges.add(elementVertex);
				fromVertex = elementGraph.getExit();
			}

			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}

		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAInputExp(AInputExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CInputExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAIntConstExp(AIntConstExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CIntConstExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAFunctionPtrCallExp(AFunctionPtrCallExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CFunctionPtrCallExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getExp());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{

			Graph fromGraph = getGraph(node.getExp());
			Vertex fromVertex = fromGraph.getExit();
			
			ArrayList<Node> sub_nodes = new ArrayList<Node>(node.getArgs());

			for (Node sub_node : sub_nodes) {
				Graph elementGraph = getGraph(sub_node);
				Vertex elementVertex = elementGraph.getEntry();
				fromVertex.edges.add(elementVertex);
				fromVertex = elementGraph.getExit();
			}

			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}

		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inABlockStm(ABlockStm node) {
		Graph nodeGraph = getGraph(node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getBlock());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getBlock());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAWhileStm(AWhileStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CWhileStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getCondition());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("true");
			
			Graph toGraph = getGraph(node.getBody());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("false");
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getBody());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAAssignVarStm(AAssignVarStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CAssignVarStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getExp());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getExp());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAIfThenElseStm(AIfThenElseStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CIfThenElseStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getCondition());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("true");
			
			Graph toGraph = getGraph(node.getThenBody());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getCondition());
			Vertex fromVertex = fromGraph.getExit("false");
			
			Graph toGraph = getGraph(node.getElseBody());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getThenBody());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getElseBody());
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAPointerExp(APointerExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CPointerExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAMallocExp(AMallocExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CMallocExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inAOutputStm(AOutputStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new COutputStmPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getExp());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getExp());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inADeclStm(ADeclStm node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CDeclStmPoint(node, "")), node);
		
		{

			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			ArrayList<Node> sub_nodes = new ArrayList<Node>(node.getDecl());

			for (Node sub_node : sub_nodes) {
				Graph elementGraph = getGraph(sub_node);
				Vertex elementVertex = elementGraph.getEntry();
				fromVertex.edges.add(elementVertex);
				fromVertex = elementGraph.getExit();
			}

			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}

		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
			
	public @Override void inABinopExp(ABinopExp node) {
		Graph nodeGraph = getGraph(node);
		
		Graph thisGraph = filterGraph(new ProgramPointGraph(node, new CBinopExpPoint(node, "")), node);
		
		{
		
			Vertex fromVertex = nodeGraph.getInnerEntry();
			
			Graph toGraph = getGraph(node.getLeft());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getLeft());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = getGraph(node.getRight());
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = getGraph(node.getRight());
			Vertex fromVertex = fromGraph.getExit();
			
			Graph toGraph = thisGraph;
			Vertex toVertex = toGraph.getEntry();
			
			fromVertex.edges.add(toVertex);
		}
		
		{
		
			Graph fromGraph = thisGraph;
			Vertex fromVertex = fromGraph.getExit();
			
			Vertex toVertex = nodeGraph.getInnerExit();
			
			fromVertex.edges.add(toVertex);
		}
		
	}
				
	/**
	 * Creates a collection of control flow graphs based on a traversal starting
	 * in {@code node}.
	 * @param node the node in which the traversal starts
	 * @return a collection of control flow graphs based on {@code node}
	 */
	public static ICFGCollection createCFGCollection(Node node) {
		CFGBuilder builder = new CFGBuilder();
		node.apply(builder);
		return builder.cfgCollection;
	}
	
	/**
	 * Creates a collection of control flow graphs based on a traversal starting
	 * in {@code node} using {@code filter} to decide which program point such
	 * appear in the resulting control flow graphs.
	 * @param node the node in which the traversal starts
	 * @param filter the {@link NodeFilter} used in the construction
	 * @return a collection of control flow graphs based on {@code node}
	 */
	public static ICFGCollection createCFGCollection(Node node, NodeFilter filter) {
		CFGBuilder builder = new CFGBuilder(filter);
		node.apply(builder);
		return builder.cfgCollection;
	}
}
