/* This file was generated by SableCC (http://www.sablecc.org/). */
 
package tip.cfg;


import tip.node.*;
import tip.analysis.*;
import java.util.*;

/**
 * {@code ICFG} defines the interface for a control flow graph.
 *
 * @author Johnni Winther, jw@brics.dk
 */
public interface ICFG {
	/**
	 * Returns the name of the control flow graph
	 * @return the name of the control flow graph
	 */
	public String getName();
	
	/**
	 * Adds the {@link IProgramPoint} {@code point} to the set
	 * of program points associated with this {@link ICFG}.
	 * Predecessors and successors are <em>not</em> updated.
	 * @param point the {@link IProgramPoint} to be added to this {@link ICFG}
	 */
	public void add(IProgramPoint point);
	
	/**
	 * Removes the {@link IProgramPoint} {@code point} from the set
	 * of program points associated with this {@link ICFG}.
	 * Predecessors and successors are <em>not</em> updated.
	 * @param point the {@link IProgramPoint} to be removed from this {@link ICFG}
	 */
	public void remove(IProgramPoint point);
	
	/**
	 * Returns the collection of program points in the control flow graph.
	 * The program points in this control flow graph are those reachable
	 * from the entry node by the successor relation.
	 * @return the collection of program points in the control flow graph
	 */
	public Collection<IProgramPoint> getProgramPoints();
	
	/**
	 * Returns {@code true} if this {@link ICFG} contains the {@link IProgramPoint} {@code point}.
	 * @param point the {@link IProgramPoint} in question
	 * @return {@code true} if this {@link ICFG} contains the {@link IProgramPoint} {@code point}
	 */
	public boolean containsProgramPoint(IProgramPoint point);
	
	/**
	 * Returns the entry point of the control flow graph.
	 * @return the entry point of the control flow graph
	 */
	public IProgramPoint getEntry();

	/**
	 * Returns the exit point of the control flow graph.
	 * @return the exit point of the control flow graph
	 */
	public IProgramPoint getExit();
	
	/**
	 * Replaces {@code point} with {@code cfg} in this control flow graph.
	 * That is, {@code point} is removed from this control flow graph, 
	 * all predecessors of {@code point} are set to have all successors
	 * of the entry point in {@code cfg} as their successor instead of 
	 * {@code point} and all successors of {@code point} are set to have
	 * all predecessors of the exit point in {@code cfg} as their predecessor
	 * instead of {@code point}. All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param point the replaced point
	 * @param cfg the replacing control flow graph
	 */
	public void replaceByCFG(IProgramPoint point, ICFG cfg);

	/**
	 * Prefixes {@code cfg} in this control flow graph. That is, 
	 * all successors of the entry point in {@code cfg} are set to be
	 * successors of the entry point in this control flow graph, and
	 * all successors of the entry point in this control flow graph
	 * are set as successors of the exit point in {@code cfg}.
	 * All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param cfg the prefixed cfg
	 */
	public void prefixCFG(ICFG cfg);
	
	/**
	 * Suffixes {@code cfg} in this control flow graph. That is, 
	 * all predecessors of the exit point in {@code cfg} are set to be
	 * predecessors of the exit point in this control flow graph, and
	 * all predecessors of the exit point in this control flow graph
	 * are set as predecessors of the entry point in {@code cfg}.
	 * All program point in {@code cfg}, except the
	 * entry and exit points, are added to this control flow graph.
	 * @param cfg the suffixed cfg
	 */
	public void suffixCFG(ICFG cfg);
	
	/**
	 * Replaces {@code oldPoint} with {@code newPoint} in this control flow graph.
	 * Successors and predecessors are transferred from {@code oldPoint} to 
	 * {@code newPoint}. {@code oldPoint} is removed from this control flow graph
	 * {@code newPoint} is added to this control flow graph.
	 * @param oldPoint the replaced point
	 * @param newPoint the replacing point
	 */
	public void replacePoint(IProgramPoint oldPoint, IProgramPoint newPoint);
	
	/**
	 * Removes {@code point} from this control flow graph, updating the
	 * predecessors and successors of {@code point} to point to eachother.
	 * @param point the removed point
	 */
	public void removePoint(IProgramPoint point);
	
	/**
	 * Inserts {@code newPoint} in the control flow graph after {@code point}.
	 * That is, all successors of {@code point} are set as successors of 
	 * {@code newPoint} and {@code newPoint} is set as the sole successor
	 * of {@code point}
	 * @param point the point in this control flow graph after which {@code newPoint} is inserted
	 * @param newPoint the inserted point
	 */
	public void insertAfter(IProgramPoint point, IProgramPoint newPoint);
	
	/**
	 * Inserts {@code newPoint} in the control flow graph before {@code point}.
	 * That is, all predecessors of {@code point} are set as predecessors of 
	 * {@code newPoint} and {@code newPoint} is set as the sole predecessor
	 * of {@code point}
	 * @param point the point in this control flow graph before which {@code newPoint} is inserted
	 * @param newPoint the inserted point
	 */
	public void insertBefore(IProgramPoint point, IProgramPoint newPoint);
	
	/**
	 * Cleans the control flow graph by removing all predecessors and successors
	 * of the program points in the program point set of this control flow graph
	 * which are not in the program point set of this control flow graph. This
	 * may be necessary for analyses to work correctly after having added 
	 * points to the control flow graph from other control flow graphs, 
	 * as after {@link #replaceByCFG(IProgramPoint,ICFG)},
	 * {@link #prefixCFG(ICFG)}, {@link #suffixCFG(ICFG)}, 
	 * {@link #replacePoint(IProgramPoint,IProgramPoint)},
	 * {@link #insertAfter(IProgramPoint,IProgramPoint)}, and
	 * {@link #insertBefore(IProgramPoint,IProgramPoint)}.
	 */
	public void clean();
	
	/**
	 * Returns a collection of descendants to the nodes associated to the program
	 * points in this control flow graph. Only descendants that are subtypes of
	 * {@code classType} are returned.
	 * @param classType the supertype of returned descendants
	 * @return a collection of descendants to the associated nodes
	 */
	public <T extends Node> Collection<T> getNodeDescendants(Class<T> classType);
	
	/**
	 * Returns a collection of descendants to the nodes associated to the program
	 * points in this control flow graph. Only descendants that are accepted by
	 * the {@link NodeFilter} {@code filter} are returned.
	 * @param filter the {@link NodeFilter} used for filtering nodes
	 * @return a collection of descendants to the associated nodes
	 */
	public <T extends Node> Collection<T> getNodeDescendants(NodeFilter<T> filter);
	
	/**
	 * Adds children to the nodes associated to the program
	 * points in this control flow graph to {@code collection}.
	 * Only descendants that are accepted by the {@link NodeFilter} {@code filter}
	 * are returned.
	 * @param collection the collection to which the descendants are added
	 * @param filter the {@link NodeFilter} used for filtering nodes
	 */
	public <T extends Node> void getNodeDescendants(Collection<T> collection, NodeFilter<T> filter);
	
	/**
	 * Applies {@code analysis} to all nodes associated with program points
	 * in this control flow graph.
	 * @param analysis the {@link Analysis} applied to the associated nodes
	 */
	public void apply(Analysis analysis);
	
	/**
	 * Applies the {@link Question} {@code caller} and {@code question} to all nodes
	 * associated with program points in this control flow graph.
	 * @param caller the {@link Question} applied to the associated nodes
	 * @param question the provided question
	 */
	public <Q> void apply(Question<Q> caller, Q question);
		
	/**
	 * Returns a deep clone of the ICFG with respect to program points,
	 * i.e. program points are cloned but the associated AST nodes are not.
	 * @return a deep clone of the ICFG
	 */
	public ICFG clone();
}
