/* This file was generated by SableCC (http://www.sablecc.org/). */
 
package tip.cfg;

import java.util.*;

import tip.analysis.*;
import tip.node.*;

/**
 * {@code IProgramPoint} defines the interface for a program point in a control
 * flow graph.
 *
 * @author Johnni Winther, jw@brics.dk
 */
public interface IProgramPoint {
	
	/**
	 * Calls the appropriate {@code case}-method on {@code analysis} and returns
	 * the result.
	 * @return the result of calling the appropriate {@code case}-method on
	 * {@code analysis}
	 */
	public boolean apply(ICFGAnalysis analysis);
	
	/**
	 * Adds {@code predecessor} to the set of predecessors of this program point.
	 * The program point as implicitly added to the successor set of the the
	 * predecessor. A program point can only be predecessor of a program point once.
	 * @param predecessor the program point added to the predecessor set
	 */
	public void addPredecessor(IProgramPoint predecessor);
	
	/**
	 * Removes {@code predecessor} from the set of predecessors of this program point.
	 * The program point as implicitly removed from the successor set of the the
	 * predecessor.
	 * @param predecessor the program point removed from the predecessor set
	 */
	public void removePredecessor(IProgramPoint predecessor);
	
	/**
	 * Adds {@code successor} to the set of successors of this program point.
	 * The program point as implicitly added to the predecessor set of the the
	 * successor. A program point can only be successor of a program point once.
	 * @param successor the program point added to the successor set
	 */
	public void addSuccessor(IProgramPoint successor);
	
	/**
	 * Removes {@code successor} to the set of successors of this program point.
	 * The program point as implicitly removes from the predecessor set of the the
	 * successor.
	 * @param successor the program point removed from the successor set
	 */
	public void removeSuccessor(IProgramPoint successor);
	
	/**
	 * Returns the set of predecessors to this program point.
	 * @return the set of predecessors to this program point
	 */
	public Set<IProgramPoint> getPredecessors();
	
	/**
	 * Returns the set of successors to this program point.
	 * @return the set of successors to this program point
	 */
	public Set<IProgramPoint> getSuccessors();
	
	/**
	 * Returns the set of program points reachable from this
	 * point following at most {@code distance} successors
	 * or predecessors.
	 * @param distance the maximum distance of points returned
	 * @return the set of program points in the {@code distance}-vicinity of this point
	 */
	public Set<IProgramPoint> getVicinity(int distance);
				
	/**
	 * Returns the {@link Node} in the AST associated with this program point.
	 * @return the {@link Node} in the AST associated with this program point
	 */
	public Node getNode();
	
	/**
	 * Returns a textual representation of this program point using the 
	 * {@link Answer} {@code nodeText}.
	 * @param nodeText the {@link Answer} used
	 * @return a textual representation of this program point
	 */
	public String getNodeText(Answer<String> nodeText);
	
	/**
	 * Returns an answer by calling the visitor {@code caller} with this program
	 * point.
	 * @param caller the {@link Answer} used
	 * @return the answer as returned from {@code caller} 
	 */
	public <A> A apply(CFGAnswer<A> caller);
	
	/**
	 * Returns an answer by calling the visitor {@code caller} with this program
	 * point and the {@code question}.
	 * @param caller the {@link QuestionAnswer} used
	 * @param question the question used
	 * @return the answer as returned from {@code caller} on {@code question}
	 */
	public <Q,A> A apply(CFGQuestionAnswer<Q,A> caller, Q question);
	
	/**
	 * Returns a clone of the program point with no predecessors or successors.
	 * @return a clone of the program point with no predecessors or successors
	 */
	public IProgramPoint clone();
}
