/* This file was generated by SableCC (http://www.sablecc.org/). */

package tip.node;

import java.util.*;
import tip.analysis.*;

/**
 * {@code Node} is the superclass of all nodes in the AST.
 */
@SuppressWarnings("nls")
public abstract class Node implements Cloneable, NodeInterface {
	private Node parent;
	
	public @Override abstract Node clone();
	public abstract Node clone(Map<Node,Node> oldToNewMap);
	/**
	 * Returns the {@link NodeEnum} corresponding to the
	 * type of this {@link Node} node.
	 * @return the {@link NodeEnum} for this node
	 */
	public abstract NodeEnum kindNode();
	
	/**
	 * Returns the parent node of this node.
	 * @return the parent node of this node
	 */
	public Node parent() {
		return this.parent;
	}
	
	/**
	 * Sets the parent node of this node.
	 * @param parent the new parent node of this node
	 */
	void parent(@SuppressWarnings("hiding") Node parent) {
		this.parent = parent;
		if (this.parent == null) {
			setOptionalInvariant(false);
		} else {
			setOptionalInvariant(this.parent.isOptionalInvariant());
		}
	}
	
	/**
	 * Removes the {@link Node} {@code child} as a child of this node.
	 * @param child the child node to be removed from this node
	 * @throws RuntimeException if {@code child} is not a child of this node
	 */
	abstract void removeChild(Node child);
	
	/**
	 * Replaces the {@link Node} {@code oldChild} child node of this node
	 * with the {@link Node} {@code newChild}.
	 * @param oldChild the child node to be replaced
	 * @param newChild the new child node of this node
	 * @throws RuntimeException if {@code oldChild} is not a child of this node
	 */
	abstract void replaceChild(Node oldChild, Node newChild);
	
	/**
	 * Replaces this node by {@code node} in the AST. If this node has no parent
	 * node, this results in a {@link NullPointerException}.
	 * The replacing {@code node} is removed from its previous parent.
	 * @param node the node replacing this node in the AST
	 */
	public void replaceBy(Node node) {
		this.parent.replaceChild(this, node);
	}
	
	/**
	 * Returns a textual representation of {@code node}. If {@code node} is {@code null}
	 * an empty string is returned.
	 * @param node the node for which a textual representation is returned
	 * @return a textual representation of {@code node}
	 */
	protected String toString(Node node) {
		if(node != null) {
			return node.toString();
		}
		return "";
	}
	
	/**
	 * Returns a textual representation of {@code list}.
	 * @param list the list for which a textual representation is returned
	 * @return a textual representation of {@code list}
	 */
	protected String toString(List<?> list) {
		StringBuffer s = new StringBuffer();
		for (Object elem : list) {
			s.append(elem);
		}
		return s.toString();
	}
	
	/**
	 * Returns a deep clone of {@code node} or {@code null} if {@code node} is {@code null}.
	 * @param node the node which is cloned
	 * @return a deep clone of {@code node}
	 */
	@SuppressWarnings("unchecked")
	protected <T extends Node> T cloneNode(T node) {
		if(node != null) {
			return (T) node.clone();
		}
		return null;
	}
	
	/**
	 * Returns a deep clone of {@code node} or {@code null} if {@code node} is {@code null}.
	 * The old node-new node relation is put into {@code oldToNewMap}.
	 * @param node the node which is cloned
	 * @param oldToNewMap the map filled with the old node-new node relation
	 * @return a deep clone of {@code node}
	 */
	@SuppressWarnings("unchecked")
	protected <T extends Node> T cloneNode(T node, java.util.Map<Node,Node> oldToNewMap) {
		if(node != null) {
			T clone = (T) node.clone(oldToNewMap);
			oldToNewMap.put(node,clone);
			return clone;
		}
		return null;
	}
	
	/**
	 * Returns a deep clone of {@code list}.
	 * @param list the list which is cloned
	 * @return a deep clone of {@code list}
	 */
	@SuppressWarnings("unchecked")
	protected <T extends Node> List<T> cloneList(List<T> list) {
		List<T> clone = new LinkedList<T>();
		for(T n : list) {
			clone.add((T) n.clone());
		}
		return clone;
	}
	
	/**
	 * Returns a deep clone of {@code list}.
	 * The old node-new node relations are put into {@code oldToNewMap}.
	 * @param list the list which is cloned
	 * @param oldToNewMap the map filled with the old node-new node relation
	 * @return a deep clone of {@code list}
	 */
	@SuppressWarnings("unchecked")
	protected <T extends Node> List<T> cloneList(List<T> list, java.util.Map<Node,Node> oldToNewMap) {
		List<T> clone = new LinkedList<T>();
		for(T n : list) {
			T cloneNode = (T) n.clone(oldToNewMap);
			oldToNewMap.put(n, cloneNode);
			clone.add(cloneNode);
		}
		return clone;
	}
	
	/**
	 * Returns the nearest ancestor of this node (including itself)
	 * which is accepted by the {@link NodeFilter} {@code filter}.
	 * Traversal is stopped if the node is guarded by
	 * the {@link NodeFilter} {@code filter}.
	 * @param filter the {@link NodeFilter} used
	 * @return the nearest ancestor of this node
	 */
	@SuppressWarnings("unchecked")
	public <T extends NodeInterface> T getAncestor(NodeFilter<T> filter) {
		Node n = this;
		do {
			if (filter.accept(n)) return (T)n;
			if (filter.guard(n)) return null;
			n = n.parent();
		} while (n != null);
		return null;
	}
	
	/**
	 * Returns the nearest ancestor of this node (including itself)
	 * which is a subclass of {@code classType}.
	 * @param classType the superclass used
	 * @return the nearest ancestor of this node
	 */
	public <T extends NodeInterface> T getAncestor(Class<T> classType) {
		Node n = this;
		while (!classType.isInstance(n)) {
			n = n.parent();
			if (n == null) return null;
		}
		return classType.cast(n);
	}
	
	/**
	 * Returns the nearest ancestor of this node (including itself)
	 * which is a subclass of {@code classType}. Traversal is stopped
	 * if a subclass of {@code guardClass} is found.
	 * @param classType the superclass used for filtering
	 * @param guardClass the superclass used for guarding
	 * @return the nearest ancestor of this node
	 */
	public <T extends NodeInterface> T getAncestor(Class<T> classType, Class<? extends NodeInterface> guardClass) {
		Node n = this;
		do {
			if (classType.isInstance(n)) return classType.cast(n);
			if (guardClass.isInstance(n)) return null;
			n = n.parent();
		} while (n != null);
		return null;
	}
	
	/**
	 * Adds all descendants of this {@link Node} node (including the node itself) that are
	 * accepted by the {@link NodeFilter} {@code filter} to {@code collection}.
	 * @param collection the collection to which the descendants are added
	 * @param filter the {@link NodeFilter} used
	 */
	public abstract <T extends NodeInterface> void getDescendants(Collection<T> collection, NodeFilter<T> filter);
	
	/**
	 * Returns the collection of descendants to this node (including the node
	 * itself) which are accepted by the {@link NodeFilter} {@code filter}.
	 * Descandants of nodes which are guarded by the {@link NodeFilter} {@code filter}
	 * are not included.
	 * @param filter the {@link NodeFilter} used
	 * @return a collection of the accepted descendants
	 */
	public <T extends NodeInterface> Collection<T> getDescendants(NodeFilter<T> filter) {
		List<T> list = new LinkedList<T>();
		getDescendants(list, filter);
		return list;
	}
	
	/**
	 * Returns the collection of descendants to this node (including itself)
	 * which are subtypes of {@code classType}.
	 * @param classType the supertype for all collected descendants
	 * @return a collection of the accepted nodes
	 */
	public <T extends NodeInterface> Collection<T> getDescendants(Class<T> classType) {
		return getDescendants(new SubTypeFilter<T>(classType));
	}
	
	/**
	 * Returns the collection of descendants to this node (including itself)
	 * which are subtypes of {@code classType}. Descendants of subtypes of
	 * {@code guardClass} are not included.
	 * @param classType the supertype for all collected descendants
	 * @return a collection of the accepted nodes
	 */
	public <T extends NodeInterface> Collection<T> getDescendants(Class<T> classType, Class<? extends NodeInterface> guardClass) {
		return getDescendants(new GuardedSubTypeFilter<T>(classType, guardClass));
	}
	
	/**
	 * Adds all children of this {@link Node} node that are
	 * accepted by the {@link NodeFilter} {@code filter} to {@code collection}.
	 * @param collection the collection to which the children are added
	 * @param filter the {@link NodeFilter} used
	 */
	public abstract <T extends NodeInterface> void getChildren(Collection<T> collection, NodeFilter<T> filter);
	
	/**
	 * Returns the collection of children to this node which are accepted by the
	 * {@link NodeFilter} {@code filter}.
	 * @param filter the {@link NodeFilter} used
	 * @return a collection of the accepted children
	 */
	public <T extends NodeInterface> List<T> getChildren(NodeFilter<T> filter) {
		List<T> list = new LinkedList<T>();
		getChildren(list, filter);
		return list;
	}
	
	/**
	 * Returns the collection of children to this node which are subtypes of
	 * {@code classType}.
	 * @param classType the supertype for all collected children
	 * @return a collection of the accepted nodes
	 */
	public <T extends NodeInterface> List<T> getChildren(Class<T> classType) {
		return getChildren(new SubTypeFilter<T>(classType));
	}
	
	/**
	 * Applies the {@link Analysis} visitor to this node.
	 * @param caller the {@link Analysis} to which this node is applied
	 */
	public abstract void apply(Analysis caller);
	
	/**
	 * Returns the answer for {@code caller} by applying this node to the
	 * {@link Answer} visitor.
	 * @param caller the {@link Answer} to which this node is applied
	 * @return the answer as returned from {@code caller}
	 */
	public abstract <A> A apply(Answer<A> caller);
	
	/**
	 * Applies this node to the {@link Question} visitor {@code caller}.
	 * @param caller the {@link Question} to which this node is applied
	 * @param question the question provided to {@code caller}
	 */
	public abstract <Q> void apply(Question<Q> caller, Q question);
	
	/**
	 * Returns the answer for {@code answer} by applying this node with the
	 * {@code question} to the {@link QuestionAnswer} visitor.
	 * @param caller the {@link QuestionAnswer} to which this node is applied
	 * @param question the question provided to {@code answer}
	 * @return the answer as returned from {@code answer}
	 */
	public abstract <Q,A> A apply(QuestionAnswer<Q,A> caller, Q question);
	
	protected boolean is_optional_invariant;
	public boolean isOptionalInvariant() {
		return is_optional_invariant;
	}
	
	public void setOptionalInvariant(boolean value) {
		if (is_optional_invariant != value) {
			is_optional_invariant = value;
			checkOptionalInvariant();
		}
	}
	
	public abstract void checkOptionalInvariant();
	
}
