package tip.node;

import tip.analysis.*;
import java.util.*;

public abstract class RoutingAutomaton<S extends Node, T extends Node> {
	static class Transition {
		final RoutingSymbol symbol;
		final int[] sourceStates;
		Transition(RoutingSymbol symbol, int[] sourceStates) {
			this.symbol = symbol;
			this.sourceStates = sourceStates;
		}
	}
	static class Pair {
		final int state;
		final Node  node;
		Pair(int state, Node node) {
			this.state = state;
			this.node = node;
		}
		public boolean equals(Object o) {
			if (o instanceof Pair) {
				return state == ((Pair)o).state && node == ((Pair)o).node;
			}
			return false;
		}
		public int hashCode() {
			return 13*state + 17*node.hashCode();
		}
	}
	static BackwardStepRelation backwardStepRelation = new BackwardStepRelation();
	final Transition[][] transitions;
	final int initialState;
	final int[] acceptStates;
	RoutingAutomaton(Transition[][] transitions, int initialState, int[] acceptStates) {
		this.transitions = transitions;
		this.initialState = initialState;
		this.acceptStates = acceptStates;
	}
	void updateRoutingExpressions(Node root) {
		Collection<S> sourceNodes = getSourceNodes(root);
		if (!sourceNodes.isEmpty()) {
			Map<Pair,T> table = new HashMap<Pair,T>();
			LinkedList<Pair> queue = new LinkedList<Pair>();
			for (T targetNode : getTargetNodes(root)) {
				for (int acceptState : acceptStates) {
					Pair pair = new Pair(acceptState, targetNode);
					table.put(pair, targetNode);
					queue.add(pair);
				}
			}
			while (!queue.isEmpty()) {
				Pair pair = queue.removeFirst();
				T target = table.get(pair);
				for (Transition transition : transitions[pair.state]) {
					for (Node sourceNode : pair.node.apply(backwardStepRelation, transition.symbol)) {
						for (int sourceState : transition.sourceStates) {
							Pair sourcePair = new Pair(sourceState, sourceNode);
							if (!table.containsKey(sourcePair)) {
								table.put(sourcePair, target);
								queue.add(sourcePair);
							}
						}
					}
				}
			}
			for (S sourceNode : sourceNodes) {
				updateRoutingExpression(sourceNode, table.get(new Pair(initialState, sourceNode)));
			}
		}
	}
	abstract Collection<S> getSourceNodes(Node root);
	abstract Collection<T> getTargetNodes(Node root);
	abstract void updateRoutingExpression(S sourceNode, T targetNode);
	public static void updateTree(Node root) {
	}
}
