package tip.util;

/**
 * {@code Equalitor} defines an interface for an
 * abstraction of the {@code equals} and {@code hashCode}
 * methods.  
 * 
 * @author Johnni Winther, jw@brics.dk
 * 
 * @param <T> the type of elements
 */
public interface Equalitor<T> {
	/**
	 * Returns {@code true} if the elements {@code a} and {@code b} are equal
	 * according to this {link Equalitor}. If the elements are equal, then
	 * we must have {@code hashCode(a) == hashCode(b)}. The method must handle
	 * the case where {@code a} or {@code b} is {@code null}.
	 * 
	 * The equality relation must atleast be reflexive in the object reference,
	 * i.e. for all elements {@code e} we must have {@code equals(e,e)==true}.
	 * 
	 * @param a	the first element
	 * @param b the second element
	 * @return {@code true} if {@code a} and {@code b} are equal
	 */
	public boolean equals(T a, T b);
	
	/**
	 * Returns the hashcode for the element {@code a} according to this 
	 * {@link Equalitor}. If {@link #equals(Object,Object)} returns 
	 * {@code true}, for two elements {@code a} and {@code b}, then for 
	 * {@code hashCode} we must have {@code hashCode(a) == hashCode(b)}.
	 * The method must handle the case where {@code a} is {@code null}.
	 * @param a the element
	 * @return the hashcode of {@code a}.
	 */
	public int hashCode(T a);
}
