
package tip.util;

/**
 * {@code IdentityEqualitor} implements the {@link Equalitor} interface by
 * calling the {@link java.lang.Object#equals(Object)} and 
 * {@link java.lang.Object#hashCode()} methods.
 * 
 * @author Johnni Winther, jw@brics.dk
 * 
 * @param <T> the type for which the equalitor applies
 */
public class IdentityEqualitor<T> implements Equalitor<T> {
	/**
	 * {@inheritDoc}
	 */
	public boolean equals(T a, T b) {
		return a == null ? b == null : a.equals(b);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int hashCode(T a) {
		return a == null ? 0 : a.hashCode();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object o) {
		if (o == null) return false;
		return getClass().equals(o.getClass());
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int hashCode() {
		return getClass().hashCode();
	}
}
