package tip;

import tip.node.*;

/**
 * Methods and intertype declarations having to do with error handling.
 */
public class Errors {
	/** Has any error occurred yet? */
	private static boolean any_errors = false;

	/**
	 * Report an error at a specific position in a source file.
	 * 
	 * @param pos
	 *            the position at which the error occurred
	 * @param message
	 *            the error message
	 */
	public static void error(Token pos, String message) {
		if (pos == null) {
			throw new InternalCompilerError(
					"Error position was null for error: " + message);
		}
		System.err.println("Error: " + "'" + message + "'" + " at "
				+ pos.getLine() + "." + pos.getPos() + ": ");

		any_errors = true;
	}

	/**
	 * Report an error at a specific position in a source file and stop the
	 * compilation.
	 * 
	 * @param pos
	 *            the position at which the error occurred
	 * @param message
	 *            the error message
	 */
	public static void fatalError(Token pos, String message) {
		error(pos, message);
		throw new SourceError();
	}

	/**
	 * Report an error not associated with a syntax node, such as a file error
	 * or parse error.
	 * 
	 * @param message
	 *            the error message
	 */
	public static void errorMessage(String message) {
		System.err.println(message);
		any_errors = true;
	}

	/**
	 * Check if any errors have occurred yet. Called between each compiler phase
	 * to stop the compilation if errors occurred during the previous phase.
	 * 
	 * @exception tip.SourceError
	 *                if any error has occurred.
	 */
	public static void check() {
		if (any_errors) {
			throw new SourceError();
		}
	}

	/** Reset the error flag for a fresh compilation */
	public static void reset() {
		any_errors = false;
	}
}
