package tip;

import tip.node.*;

/**
 * A number of convenient enums and intertype methods for switching on and
 * extracting information from various AST nodes.
 * 
 * @author Poul Hansen
 * @author Johnni Winther
 */
public class Util {
	/**
	 * Search up the AST for an ancestor node of a specific type.
	 * 
	 * @param start
	 *            the node at which to start the search
	 * @param cl
	 *            the class of the node to search for
	 * @return the closest ancestor (including the start node itself) of the
	 *         specified type, or <code>null</code> if no ancestor of the start
	 *         node has the specified type.
	 */
	public static <T> T findNode(Node start, Class<T> cl) {
		Node n = start;
		while (!cl.isInstance(n)) {
			n = n.parent();
			if (n == null) {
				return null;
			}
		}
		return cl.cast(n);
	}

	/**
	 * An enum for binary operations. Each subclass of {@link tip.node.PBinop}
	 * has a kind which is one of these.
	 */
	/*
	 * public static enum Binop { PLUS, MINUS, TIMES, DIVIDE, EQ, GT } public
	 * abstract Binop PBinop.kind(); public Binop APlusBinop.kind() { return
	 * Binop.PLUS; } public Binop AMinusBinop.kind() { return Binop.MINUS; }
	 * public Binop ATimesBinop.kind() { return Binop.TIMES; } public Binop
	 * ADivideBinop.kind() { return Binop.DIVIDE; } public Binop AEqBinop.kind()
	 * { return Binop.EQ; } public Binop AGtBinop.kind() { return Binop.GT; }
	 */
	/**
	 * The token which constitutes this binary operator in the source.
	 */
	// public String PBinop.opName() { return getToken().getText(); }
	/**
	 * getPrecedence is used by NodeText to add parentheses where needed.
	 */
	/*
	 * public abstract int PExp.getPrecedence(); public int
	 * ABinopExp.getPrecedence() { switch (getBinop().kind()) { case TIMES: case
	 * DIVIDE: return 2; case PLUS: case MINUS: return 3; case EQ: case GT:
	 * return 4; } throw new InternalCompilerError("Unknown binop : " +
	 * getBinop()); } public int APointerExp.getPrecedence() { return 1; }
	 * public int ADerefExp.getPrecedence() { return 1; } public int
	 * AIntConstExp.getPrecedence() { return 0; } public int
	 * AFunctionCallExp.getPrecedence() { return 0; } public int
	 * AFunctionPtrCallExp.getPrecedence() { return 0; } public int
	 * AVarExp.getPrecedence() { return 0; } public int
	 * AInputExp.getPrecedence() { return 0; } public int
	 * AMallocExp.getPrecedence() { return 0; } public int
	 * ANullExp.getPrecedence() { return 0; }
	 */
	/**
	 * Returns the integer value of an AIntConstExp.
	 */
	/*
	 * public Integer AIntConstExp.getValue() { return
	 * Integer.valueOf(getIntegerConst().getText()); }
	 */

	/**
	 * Methods present in all subtypes are declared in the supertype for
	 * convenience.
	 */
	// public abstract Token PBinop.getToken();
	// public abstract TIdentifier PDecl.getName();
	// public abstract Token PStm.getToken();
}
