package tip.cfg;

import java.util.*;

/**
 * {@code BackwardsStrategy} defines the methods for retrieving the dependency
 * set and joint set for a program point in a backwards analysis. This strategy
 * is used for analyses that compute information about <em>future</em>
 * behaviour.
 * 
 * @author Johnni Winther, jw@brics.dk
 */
public class BackwardsStrategy implements DependencyStrategy {
	/**
	 * Returns the set of {@link IProgramPoint}s that are dependent on {@code
	 * point}. This is the set of predecessors to {@code point}.
	 * 
	 * @param point
	 *            the {@link IProgramPoint} for which the dependency set is
	 *            returned
	 * @return the set of {@link IProgramPoint}s dependent on {@code point}
	 */
	public Set<IProgramPoint> getDependencySet(IProgramPoint point) {
		return point.getPredecessors();
	}

	/**
	 * Returns the set of {@link IProgramPoint}s that are {@code point} depends
	 * upon. This is the set of successors to {@code point}.
	 * 
	 * @param point
	 *            the {@link IProgramPoint} for which the join set is returned
	 * @return the set of {@link IProgramPoint}s {@code point} dependents upon
	 */
	public Set<IProgramPoint> getJoinSet(IProgramPoint point) {
		return point.getSuccessors();
	}
}
