package tip.cfg;

import tip.node.*;

/**
 * {@code DeclarationTarget} represents the pointer target of a declaration
 * (both function and variable declarations).
 * 
 * @author Johnni Winther, jw@brics.dk
 */
public class DeclarationTarget implements Target {
	// The target declaration
	protected PDecl decl;

	/**
	 * Creates a new {@code DeclarationTarget} pointing to {@code decl}.
	 * 
	 * @param decl
	 *            the target of this {@link DeclarationTarget}
	 */
	public DeclarationTarget(PDecl decl) {
		this.decl = decl;
	}

	/**
	 * Returns the target {@link PDecl} of this {@link DeclarationTarget}.
	 * 
	 * @return the target {@link PDecl} of this {@link DeclarationTarget}
	 */
	public PDecl getTarget() {
		return decl;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof DeclarationTarget) {
			return decl == ((DeclarationTarget) o).decl;
		}
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		return decl.hashCode() * 11;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "&" + decl.getName().getText();
	}
}
