package tip.cfg;

import java.util.*;

/**
 * The {@code DependencyStrategy} interface defines the necessary method to
 * abstract forwards and backwards dependency in a {@link FlowAnalysis}.
 * 
 * @author Johnni Winther
 */
public interface DependencyStrategy {
	/**
	 * Returns the set of {@link IProgramPoint}s that are dependent on {@code
	 * point}.
	 * 
	 * @param point
	 *            the {@link IProgramPoint} for which the dependency set is
	 *            returned
	 * @return the set of {@link IProgramPoint}s dependent on {@code point}
	 */
	public Set<IProgramPoint> getDependencySet(IProgramPoint point);

	/**
	 * Returns the set of {@link IProgramPoint}s that are {@code point} depends
	 * upon. This should be the inverse of
	 * {@link #getDependencySet(IProgramPoint)}.
	 * 
	 * @param point
	 *            the {@link IProgramPoint} for which the join set is returned
	 * @return the set of {@link IProgramPoint}s {@code point} dependents upon
	 */
	public Set<IProgramPoint> getJoinSet(IProgramPoint point);
}
