package tip.cfg;

import tip.util.Equalitor;

/**
 * {@code Lattice} defines the interface for a lattice with element type {@code
 * T}.
 * 
 * @author Johnni Winther
 * @param <T>
 *            the type of the lattice elements
 */
public interface Lattice<T> extends Equalitor<T> {

	/**
	 * Returns the smallest element of the lattice. Beware that if the lattice
	 * elements are mutable, {@code getSmallestElement} must return a cloned
	 * element.
	 * 
	 * @return the smallest element of the lattice
	 */
	public T getSmallestElement();

	/**
	 * Returns the least upper bound of the lattice elements {@code a} and
	 * {@code b}. Beware that if the lattice elements are mutable, {@code
	 * getLeastUpperBound} must return a cloned or new element. The method must
	 * handle the case where {@code a} or {@code b} are {@code null}.
	 * 
	 * @param a
	 *            the first element
	 * @param b
	 *            the second element
	 * @return the least upper bound of the first and second element
	 */
	public T getLeastUpperBound(T a, T b);
}
