package tip.cfg;

import tip.node.*;

/**
 * {@code MallocTarget} represents the pointer target of a {@code malloc}
 * expression.
 * 
 * @author Johnni Winther, jw@brics.dk
 */
public class MallocTarget implements Target {
	// The target malloc expression
	protected AMallocExp exp;

	/**
	 * Creates a new {@code MallocTarget} pointing to {@code exp}.
	 * 
	 * @param exp
	 *            the target of this {@link MallocTarget}
	 */
	public MallocTarget(AMallocExp exp) {
		this.exp = exp;
	}

	/**
	 * Returns the target {@link AMallocExp} of this {@link MallocTarget}.
	 * 
	 * @return the target {@link AMallocExp} of this {@link MallocTarget}
	 */
	public AMallocExp getTarget() {
		return exp;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof MallocTarget) {
			return exp == ((MallocTarget) o).exp;
		}
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		return exp.hashCode() * 13;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "malloc-" + exp.getMallocIndex();
	}
}
