package tip.cfg;

import java.util.*;
import tip.node.*;

/**
 * {@code PointsToMap} defines the interface for a points-to map mapping
 * variables to pointer target for a program point.
 * 
 * @author Johnni Winther, jw@brics.dk
 */
public interface PointsToMap {
	/**
	 * Returns the name of the points-to map.
	 * 
	 * @return the name of the points-to map
	 */
	public String getName();

	/**
	 * Returns the pointer targets for the variable {@code decl} at the program
	 * point {@code point}.
	 * 
	 * @param point
	 *            the program point for which the pointer targets are returned
	 * @param decl
	 *            the variable for which the pointer targets are returned
	 * @return the pointer target for the {@code decl} at {@code point}
	 */
	public Set<Target> pt(IProgramPoint point, ALocalDecl decl);

	public boolean isFlowSensitive();

	public Map<Target, Set<Target>> pt(IProgramPoint point);
}
