package tip.cfg;

/**
 * {@code WideningNarrowingAnalysis} defines the methods necessary for
 * performing a widening-narrowing analysis using {@link WorkList}.
 * 
 * @author Johnni Winther, jw@brics.dk
 * @param <VariableType>
 *            the variable type of the analysis
 */
public interface WideningNarrowingAnalysis<VariableType> extends
		FixedPointAnalysis<VariableType> {
	/**
	 * Returns {@code true} if the current analysis is in the narrowing phase.
	 * 
	 * @return {@code true} if the current analysis is in the narrowing phase
	 */
	public boolean isNarrowing();

	/**
	 * Sets the current analysis to be in the narrowing phase.
	 */
	public void setNarrowing();

	/**
	 * Returns {@code true} if the current analysis is in the widening phase.
	 * 
	 * @return {@code true} if the current analysis is in the widening phase
	 */
	public boolean isWidening();

	/**
	 * Sets the current analysis to be in the widening phase.
	 */
	public void setWidening();
}
