package tip.cfg.dfa;

import tip.node.*;
import tip.cfg.*;

public class DivisionAnalysis extends FlowAnalysis<DivisionAnalysis.Div> {
	public DivisionAnalysis() {
		super(new BackwardsStrategy(), "Division");
	}

	public enum Div implements Lattice<Div> {
		NOT_SURE("?"), NO("NO"), BOT("_");

		private String name;

		private Div(String name) {
			this.name = name;
		}

		@Override
		public String toString() {
			return name;
		}

		public Div getSmallestElement() {
			return BOT;
		}

		public Div getLeastUpperBound(Div a, Div b) {
			if (a == NOT_SURE || b == NOT_SURE) {
				return NOT_SURE;
			} else if (a == NO || b == NO) {
				return NO;
			} else {
				return BOT;
			}
		}

		public boolean equals(Div a, Div b) {
			return a == b;
		}

		public int hashCode(Div a) {
			return a.hashCode();
		}
	}

	@Override @SuppressWarnings("unused")
	protected Lattice<Div> createLattice(ICFG cfg) {
		return Div.BOT;
	}

	public @Override
	boolean defaultNode(Node node, IProgramPoint point) {
		Div join = join(point);
		if (node.getDescendants(ADivideBinop.class).size() > 0) {
			join = Div.NOT_SURE;
		}
		return setVariable(point, join);
	}

	public @Override @SuppressWarnings("unused")
	boolean caseExitPoint(Node node, ExitPoint point) {
		return setVariable(point, Div.NO);
	}

	public String getVariableString(Div variable) {
		return variable.toString();
	}
}
