package tip.cfg.dfa;

import tip.cfg.*;
import tip.node.*;
import tip.util.*;
import tip.display.*;

public class VeryBusyExpressionsAnalysis extends
		FlowAnalysis<EqualitySet<ABinopExp>> {
	public VeryBusyExpressionsAnalysis() {
		super(new BackwardsStrategy(), "Very Busy Expressions");
	}

	private NodeFilter<ABinopExp> filter = new SubTypeFilter<ABinopExp>(
			ABinopExp.class);

	@Override @SuppressWarnings("unused")
	protected Lattice<EqualitySet<ABinopExp>> createLattice(ICFG cfg) {
		return new UnionSetLattice<ABinopExp>(new ExpressionEqualitor());
	}

	public @Override @SuppressWarnings("unused")
	boolean caseExitPoint(Node node, ExitPoint point) {
		return setVariable(point, getLattice().getSmallestElement());
	}

	public @Override
	boolean caseCAssignVarStmPoint(AAssignVarStm node, CAssignVarStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		EqualitySet<ABinopExp> downarrow = getLattice().getSmallestElement();
		for (ABinopExp exp : join) {
			if (!DeclCollector.collectDecls(exp).contains(
					node.getIdentifier().getDecl())) {
				downarrow.add(exp);
			}
		}
		node.getDescendants(join, filter);
		join = downarrow;
		return setVariable(point, join);
	}

	public @Override
	boolean caseCIfThenStmPoint(AIfThenStm node, CIfThenStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		node.getCondition().getDescendants(join, filter);
		return setVariable(point, join);
	}

	public @Override
	boolean caseCIfThenElseStmPoint(AIfThenElseStm node,
			CIfThenElseStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		node.getCondition().getDescendants(join, filter);
		return setVariable(point, join);
	}

	public @Override
	boolean caseCWhileStmPoint(AWhileStm node, CWhileStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		node.getCondition().getDescendants(join, filter);
		return setVariable(point, join);
	}

	public @Override
	boolean caseCOutputStmPoint(AOutputStm node, COutputStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		node.getExp().getDescendants(join, filter);
		return setVariable(point, join);
	}

	public @Override
	boolean caseCReturnStmPoint(AReturnStm node, CReturnStmPoint point) {
		EqualitySet<ABinopExp> join = join(point);
		node.getExp().getDescendants(join, filter);
		return setVariable(point, join);
	}

	public String getVariableString(EqualitySet<ABinopExp> variable) {
		return variable.toString(new NodeText<ABinopExp>());
	}
}
