package tip.display;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

import tip.node.*;
import tip.cfg.*;
import tip.util.*;

import dk.brics.dot.*;
import dk.brics.dot.color.*;
import dk.brics.dot.html.*;
import dk.brics.dot.view.*;

public class CFGDisplay {
	public static final boolean useParser = true;

	private static NodeText<tip.node.Node> nodeText;

	public static void displayCFGs(ICFGCollection cfgList,
			NodeText<tip.node.Node> _nodeText,
			Collection<FixedPointAnalysis<?>> analyses,
			Collection<PointsToMap> pointsToMaps, int vicinity) {
		nodeText = _nodeText;

		Map<String, DotGraph> dotMap = new HashMap<String, DotGraph>();
		Map<String, JTable> resultMap = new HashMap<String, JTable>();
		EdgeAttributes edgeStyle = new EdgeAttributes();
		edgeStyle.setArrowHead(new ArrowShape());
		FontAttributes fontStyle = new FontAttributes();
		fontStyle.setFontName("Courier");
		fontStyle.setFontSize(10.0);
		NodeAttributes nodeStyle = new NodeAttributes();
		nodeStyle.setFontStyle(fontStyle);
		nodeStyle.setShape(DotShape.plaintext);
		EdgeAttributes attributes = new EdgeAttributes();
		ColorList analysisColors = X11.goldenrods;
		ColorList pointsToMapColors = X11.darkolivegreens;
		for (ICFG cfg : cfgList.getCFGs()) {
			int analysisColorIndex = 0;
			int pointsToMapColorIndex = 0;
			HashMap<IProgramPoint, Integer> pointToIntegerMap = new LinkedHashMap<IProgramPoint, Integer>();
			int pointIndex = 0;

			DotGraph graph = new DotGraph(cfg.getName(), true);
			graph.setCompound(true);
			graph.createNodeStatement(nodeStyle);
			graph.createEdgeStatement(attributes);

			if (analyses != null && analyses.size() > 0) {
				DotNode node = graph.createNode();
				HTMLTableLabel label = new HTMLTableLabel();
				label.setBorder(0);
				label.setCellBorder(1);
				label.setCellSpacing(0);
				label.setCellPadding(1);
				HTMLRow row = label.createRow();
				HTMLLabelCell cell = new HTMLLabelCell("Analyses:");
				row.addCell(cell);
				for (FixedPointAnalysis<?> analysis : analyses) {
					cell = new HTMLLabelCell(analysis.getName());
					cell.setBgColor(analysisColors.getColor(analysisColorIndex));
					row.addCell(cell);
					analysisColorIndex++;
				}
				node.setLabel(label);
			}

			if (pointsToMaps != null && pointsToMaps.size() > 0) {
				DotNode node = graph.createNode();
				HTMLTableLabel label = new HTMLTableLabel();
				label.setBorder(0);
				label.setCellBorder(1);
				label.setCellSpacing(0);
				label.setCellPadding(1);
				HTMLRow row = label.createRow();
				HTMLLabelCell cell = new HTMLLabelCell("PointsToMaps:");
				row.addCell(cell);
				for (PointsToMap pointsToMap : pointsToMaps) {
					DotColor color = pointsToMapColors.getColor(pointsToMapColorIndex);
					String name = pointsToMap.getName();
					cell = new HTMLLabelCell(name);
					cell.setBgColor(color);
					cell.setPort(name);
					row.addCell(cell);
					pointsToMapColorIndex++;
					if (!pointsToMap.isFlowSensitive()) {
						DotNode dotNode = createPointsToGraph(graph,
								pointsToMap.pt(null), name, color);
						DotEdge dotEdge = graph.createEdge(node,
								pointsToMap.getName(), dotNode, null);
						dotEdge.setDir(Dir.none);
						/*
						 * graph.createEdge( new NodeConnection(node,
						 * pointsToMap.getName()), new
						 * SubgraphConnection(subgraph) );
						 */
					}
				}
				node.setLabel(label);
			}

			Map<IProgramPoint, DotNode> pointToNodeMap = new HashMap<IProgramPoint, DotNode>();
			for (IProgramPoint point : cfg.getProgramPoints()) {
				int thisPointIndex = pointIndex++;
				pointToIntegerMap.put(point, thisPointIndex);

				DotNode node = graph.createNode();
				pointToNodeMap.put(point, node);
				HTMLTableLabel label = new HTMLTableLabel();
				label.setBorder(0);
				label.setCellBorder(1);
				label.setCellSpacing(0);
				label.setCellPadding(1);

				HTMLRow row = label.createRow();
				HTMLLabelCell indexCell = new HTMLLabelCell(""+thisPointIndex);
				indexCell.setBgColor(X11.aliceblue); row.addCell(indexCell);
				
				String labelText = point.getNodeText(nodeText);
				if (labelText.length() > 40) {
					labelText = labelText.substring(0, 37) + "...";
				}
				HTMLLabelCell cell = new HTMLLabelCell(labelText);
				cell.setPort("point");
				row.addCell(cell);
				node.setLabel(label);

				if (analyses != null) {
					analysisColorIndex = 0;
					for (FixedPointAnalysis<?> analysis : analyses) {
						Object variable = analysis.getVariable(point);
						if (variable != null) {
							String variableLabel = vicinity >= 0 ? analysis.getVariableStringForPoint(
									point, vicinity)
									: analysis.getVariableStringForPoint(point);
							HTMLLabelCell analysisCell = new HTMLLabelCell(
									variableLabel);
							row.addCell(analysisCell);
							analysisCell.setBgColor(analysisColors.getColor(analysisColorIndex));
						}
						analysisColorIndex++;
					}
				}
				if (pointsToMaps != null) {
					pointsToMapColorIndex = 0;
					for (PointsToMap pointsToMap : pointsToMaps) {
						DotColor color = pointsToMapColors.getColor(pointsToMapColorIndex);
						String name = pointsToMap.getName();
						if (pointsToMap.isFlowSensitive()) {
							DotNode dotNode = createPointsToGraph(graph,
									pointsToMap.pt(point), name, color);
							DotEdge dotEdge = graph.createEdge(node, "point",
									dotNode, null);
							dotEdge.setDir(Dir.none);
							/*
							 * graph.createEdge( new NodeConnection(node,
							 * pointsToMap.getName()), new
							 * SubgraphConnection(subgraph) );
							 */
						}
						/*
						 * Map<ALocalDecl,Set<Target>> pointsTo = new
						 * HashMap<ALocalDecl,Set<Target>>();
						 * Collection<ALocalDecl> declCollection = vicinity >= 0
						 * ?
						 * LocalDeclCollector.collectLocalDecls(point.getVicinity
						 * (vicinity)) :
						 * LocalDeclCollector.collectLocalDecls(cfg); for
						 * (ALocalDecl decl : declCollection) {
						 * pointsTo.put(decl, pointsToMap.pt(point, decl)); } if
						 * (pointsTo.size() > 0) { String variableLabel =
						 * pointsTo.toString(); HTMLLabelCell analysisCell = new
						 * HTMLLabelCell(variableLabel);
						 * row.addCell(analysisCell);
						 * analysisCell.setBgColor(pointsToMapColors
						 * .getColor(pointsToMapColorIndex)); }
						 */
						pointsToMapColorIndex++;
					}
				}

			}
			for (IProgramPoint point : cfg.getProgramPoints()) {
				DotNode from = pointToNodeMap.get(point);
				for (IProgramPoint successor : point.getSuccessors()) {
					if (cfg.containsProgramPoint(successor)) {
						DotNode to = pointToNodeMap.get(successor);
						DotEdge edge = graph.createEdge(from, "point", to,
								"point");
						edge.setEdgeAttributes(edgeStyle);
					}
				}
			}

			dotMap.put(cfg.getName(), graph);
			if (useParser) {
				PointTableModel tableModel = new PointTableModel(
						pointToIntegerMap, analyses, pointsToMaps, vicinity);
				JTable table = new JTable(tableModel,
						tableModel.getTableColumnModel());
				resultMap.put(cfg.getName(), table);
			}
		}

		JFrame frame = new JFrame("CFG Displayer");
		JTabbedPane tabbedPane = new JTabbedPane(SwingConstants.LEFT);
		for (Map.Entry<String, DotGraph> entry : dotMap.entrySet()) {
			if (!useParser) {
				tabbedPane.add(entry.getKey(), new DotPanel(entry.getValue()));
			} else {
				JSplitPane splitPane = new JSplitPane(
						JSplitPane.VERTICAL_SPLIT, new DotComponentPanel(
								entry.getValue()), new JScrollPane(
								resultMap.get(entry.getKey())));
				splitPane.setResizeWeight(0.8);
				tabbedPane.add(entry.getKey(), splitPane);
			}
		}
		// window listener so the program will die
		frame.addWindowListener(new WindowAdapter() {
			@Override
			@SuppressWarnings("unused")
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.setSize(800, 600);
		frame.getContentPane().add(tabbedPane);
		frame.setVisible(true);
		try {
			while (true) {
				Thread.sleep(1000);
			}
		} catch (InterruptedException e) {

		}

	}

	@SuppressWarnings("serial")
	private static class PointTableModel extends AbstractTableModel {
		private int rowCount;

		private int columnCount;

		private String[][] data;

		private DefaultTableColumnModel tableColumnModel = new DefaultTableColumnModel();

		public PointTableModel(Map<IProgramPoint, Integer> map,
				Collection<FixedPointAnalysis<?>> analyses,
				Collection<PointsToMap> pointsToMaps, int vicinity) {
			rowCount = map.size();
			columnCount = 1;
			TableColumn numberColumn = new TableColumn(0);
			numberColumn.setWidth(20);
			numberColumn.setHeaderValue("#");
			tableColumnModel.addColumn(numberColumn);
			if (analyses != null) {
				for (FixedPointAnalysis<?> analysis : analyses) {
					TableColumn analysisColumn = new TableColumn(columnCount++);
					analysisColumn.setHeaderValue(analysis.getName());
					tableColumnModel.addColumn(analysisColumn);
				}
			}
			if (pointsToMaps != null) {
				for (PointsToMap pointsToMap : pointsToMaps) {
					TableColumn pointsToMapColumn = new TableColumn(
							columnCount++);
					pointsToMapColumn.setHeaderValue(pointsToMap.getName());
					tableColumnModel.addColumn(pointsToMapColumn);
				}
			}

			data = new String[rowCount][columnCount];

			int rowIndex = 0;
			for (IProgramPoint point : map.keySet()) {
				int columnIndex = 0;
				data[rowIndex][columnIndex++] = map.get(point).toString();

				if (analyses != null) {
					for (FixedPointAnalysis<?> analysis : analyses) {
						data[rowIndex][columnIndex++] = vicinity >= 0 ? analysis.getVariableStringForPoint(
								point, vicinity)
								: analysis.getVariableStringForPoint(point);
					}
				}
				if (pointsToMaps != null) {
					for (PointsToMap pointsToMap : pointsToMaps) {
						Map<ALocalDecl, Set<Target>> pointsTo = new HashMap<ALocalDecl, Set<Target>>();
						for (TIdentifier id : point.apply(new SubExpressions<TIdentifier>(
								new SubTypeFilter<TIdentifier>(
										TIdentifier.class)))) {
							if (id.getDecl() instanceof ALocalDecl) {
								pointsTo.put((ALocalDecl) id.getDecl(),
										pointsToMap.pt(point,
												(ALocalDecl) id.getDecl()));
							}
						}
						data[rowIndex][columnIndex++] = pointsTo.toString();
					}

				}
				rowIndex++;
			}

		}

		public int getRowCount() {
			return rowCount;
		}

		public int getColumnCount() {
			return columnCount;
		}

		public Object getValueAt(int row, int column) {
			return data[row][column];
		}

		public TableColumnModel getTableColumnModel() {
			return tableColumnModel;
		}
	}

	public static DotNode createPointsToGraph(DotGraph graph,
			Map<Target, Set<Target>> ptMap, String name, DotColor color) {
		Subgraph subgraph = graph.createSubgraph(true);
		subgraph.setRankDir(RankDir.TB);
		subgraph.setColor(color);
		subgraph.setFilled(true);
		NodeAttributes nodeAttributes = new NodeAttributes();
		nodeAttributes.setShape(DotShape.box);
		nodeAttributes.setWidth(0.5);
		nodeAttributes.setHeight(0.25);
		DotNode dotNode = subgraph.createNode();
		dotNode.setLabel(new EscStringLabel(name));
		for (Target from : ptMap.keySet()) {
			DotNode fromNode = subgraph.getNodeForObject(from);
			fromNode.setNodeAttributes(nodeAttributes);
			fromNode.setLabel(new EscStringLabel(from.toString()));
			Set<Target> toSet = ptMap.get(from);
			for (Target to : toSet) {
				DotNode toNode = subgraph.getNodeForObject(to);
				if (toNode == null) {
					toNode = subgraph.getNodeForObject(to);
					toNode.setNodeAttributes(nodeAttributes);
					toNode.setLabel(new EscStringLabel(to.toString()));
				}
				subgraph.createEdge(fromNode, toNode);
			}
		}
		return dotNode;
	}
}
