package tip.instructions;

import java.util.*;

public abstract class Instruction {
	public int location = 0;

	/**
	 * An identification number associated with the instruction type.
	 * 
	 * @see #getID
	 */
	private int id;

	private static Map<Class<? extends Instruction>, Integer> inst_id = new HashMap<Class<? extends Instruction>, Integer>();

	private static int next_id = 0;

	public Instruction() {
		id = getID(getClass());
	}

	/**
	 * Returns an identification number associated with the instruction type.
	 * Each instruction class gets a unique ID which can be used to do efficient
	 * switching on instruction types.
	 */
	public final int getID() {
		return id;
	}

	/**
	 * Returns an identification number associated with the instruction type.
	 * Each instruction class gets a unique ID which can be used to do efficient
	 * switching on instruction types.
	 * 
	 * @param c
	 *            the instruction class to get the ID for
	 */
	public static int getID(Class<? extends Instruction> c) {
		if (inst_id.containsKey(c)) {
			return inst_id.get(c);
		} else {
			int this_id = next_id++;
			inst_id.put(c, this_id);
			return this_id;
		}
	}

	public abstract String toAsm();

	public abstract int[] genCode();

	public abstract int getCodeSize();
}
