package tip.phases;

import tip.*;
import tip.analysis.*;
import tip.node.*;
import static tip.Util.*;

/*
 * This phase to build maps from names to AST nodes where these
 * are declared.
 */
public class Environments extends DepthFirstAdapter {
	public @Override
	void inALocalDecl(ALocalDecl node) {
		PBlock block = null;
		if (node.parent() instanceof AFuncDecl) {
			AFuncDecl decl = (AFuncDecl) node.parent();
			block = (decl.getBody()).getBlock();
		} else {
			block = findNode(node, PBlock.class);
		}

		if (block != null) {
			PBlock parentBlock = block;
			do {
				if (parentBlock.getLocalEnv().containsKey(
						node.getName().getText())) {
					Errors.error(node.getName(),
							"Duplicate local variable name : "
									+ node.getName().getText());
				}
				parentBlock = findNode(parentBlock.parent(), PBlock.class);
			} while (parentBlock != null);
			block.getLocalEnv().put(node.getName().getText(), node);
		}
	}

	public @Override
	void inAFuncDecl(AFuncDecl node) {
		AFuncDecl oldNode = findNode(node, PProgram.class).getFunctionEnv().put(
				node.getName().getText(), node);
		if (oldNode != null) {
			Errors.error(node.getName(), "Duplicate function name : "
					+ node.getName().getText());
		}
	}

	private int mallocIndex = 0;

	public @Override @SuppressWarnings("unused")
	void inAProgram(AProgram node) {
		mallocIndex = 1;
	}

	public @Override
	void inAMallocExp(AMallocExp node) {
		node.setMallocIndex(mallocIndex++);
	}
}
