package tip.phases;

import tip.*;
import tip.analysis.*;
import tip.node.*;
import static tip.Util.*;

/*
 * This phase links all uses to declarations.
 */
public class NameLinking extends DepthFirstAdapter {
	// A marker that is used to determine if a local is declared before use
	// public boolean ALocalDecl.declared = false;

	public @Override
	void inALocalDecl(ALocalDecl node) {
		node.setDeclared(true);
	}

	public @Override
	void caseTIdentifier(TIdentifier name) {
		if (name.parent() instanceof ALocalDecl) {
			return;
		}

		// (1) Try local env
		PBlock block = findNode(name, PBlock.class);
		while (block != null) {
			ALocalDecl local_decl = block.getLocalEnv().get(name.getText());
			if (local_decl != null) {
				if (!local_decl.getDeclared()) {
					break;
				}

				// Make uses point to declarations
				name.setDecl(local_decl);
				return;
			}
			block = findNode(block.parent(), PBlock.class);
		}

		// (2) Try function env
		PProgram program = findNode(name, PProgram.class);
		AFuncDecl func_decl = program.getFunctionEnv().get(name.getText());
		if (func_decl != null) {
			// Make uses point to declarations
			name.setDecl(func_decl);
			return;
		}

		Errors.error(name, "Identifier '" + name.getText() + "' not declared");
	}

	/*
	 * public @Override void caseTIdentifier(TIdentifier name){ PProgram program
	 * = findNode(name, PProgram.class); if(
	 * program.decls.containsKey(name.getText()) ){ // Check that id is declared
	 * in the right function name.decl = program.decls.get(name.getText()); if(
	 * name.decl instanceof ALocalDecl ){ AFuncDecl f = findNode(name,
	 * AFuncDecl.class); if( f != findNode(name.decl.getName(),AFuncDecl.class)
	 * ){ Errors.error( name, "cannot access '"+name+"'"); } } } else {
	 * Errors.error( name,"identifier '"+name.getText()+"' not declared"); } }
	 */
	/*
	 * public void Node.getVars(Set<ALocalDecl> vars) { } public void
	 * AProgram.getVars(Set<ALocalDecl> vars) { for (PDecl decl :
	 * getFunctions()) { decl.getVars(vars); } } public void
	 * AFuncDecl.getVars(Set<ALocalDecl> vars) { for (PDecl decl : getFormals())
	 * { decl.getVars(vars); } getBody().getVars(vars); } public void
	 * ALocalDecl.getVars(Set<ALocalDecl> vars) { vars.add(this); } public void
	 * ABody.getVars(Set<ALocalDecl> vars) { getBlock().getVars(vars); } public
	 * void ABlock.getVars(Set<ALocalDecl> vars) { for (PStm stm :
	 * (List<PStm>)getStatements()) { stm.getVars(vars); } } public void
	 * ABinopExp.getVars(Set<ALocalDecl> vars) { getLeft().getVars(vars);
	 * getRight().getVars(vars); } public void
	 * APointerExp.getVars(Set<ALocalDecl> vars) { PDecl decl =
	 * getIdentifier().decl; if (decl instanceof ALocalDecl) {
	 * vars.add((ALocalDecl)decl); } } public void
	 * ADerefExp.getVars(Set<ALocalDecl> vars) { getExp().getVars(vars); }
	 * public void AFunctionCallExp.getVars(Set<ALocalDecl> vars) { for (PExp
	 * exp : (List<PExp>)getArgs()) { exp.getVars(vars); } } public void
	 * AFunctionPtrCallExp.getVars(Set<ALocalDecl> vars) {
	 * getExp().getVars(vars); for (PExp exp : (List<PExp>)getArgs()) {
	 * exp.getVars(vars); } } public void AVarExp.getVars(Set<ALocalDecl> vars)
	 * { PDecl decl = getIdentifier().decl; if (decl instanceof ALocalDecl) {
	 * vars.add((ALocalDecl)decl); } } public void
	 * AIfThenStm.getVars(Set<ALocalDecl> vars) { getCondition().getVars(vars);
	 * getBody().getVars(vars); } public void
	 * AIfThenElseStm.getVars(Set<ALocalDecl> vars) {
	 * getCondition().getVars(vars); getThenBody().getVars(vars);
	 * getElseBody().getVars(vars); } public void
	 * AWhileStm.getVars(Set<ALocalDecl> vars) { getCondition().getVars(vars);
	 * getBody().getVars(vars); } public void ADeclStm.getVars(Set<ALocalDecl>
	 * vars) { for (PDecl decl : getDecl()) { decl.getVars(vars); } } public
	 * void AReturnStm.getVars(Set<ALocalDecl> vars) { getExp().getVars(vars); }
	 * public void AOutputStm.getVars(Set<ALocalDecl> vars) {
	 * getExp().getVars(vars); } public void
	 * AAssignVarStm.getVars(Set<ALocalDecl> vars) { PDecl decl =
	 * getIdentifier().decl; if (decl instanceof ALocalDecl) {
	 * vars.add((ALocalDecl)decl); } getExp().getVars(vars); }
	 */
	/*
	 * public void AAssignPtrStm.getVars(Set<ALocalDecl> vars) {
	 * Environments.Declaration decl = getIdentifier().decl; if (decl instanceof
	 * ALocalDecl) { vars.add((ALocalDecl)decl); } getExp().getVars(vars); }
	 */
	/*
	 * public void AAssignPtrStm.getVars(Set<ALocalDecl> vars) {
	 * getLeft().getVars(vars); getRight().getVars(vars); }
	 */
}
