package tip.types;

import tip.*;

import java.util.*;

public class Function extends Type {
	private Type returnType;

	private Type[] args;

	public Function(LinkedList<Type> argTypes, Type returnType) {
		this.returnType = returnType;

		args = argTypes.toArray(new Type[argTypes.size()]);
	}

	public Type getReturnType() {
		return returnType;
	}

	public Type[] getArgTypes() {
		return args;
	}

	@Override
	public void unify(Type other) {
		Type r1 = find();
		Type r2 = other.find();

		if (r1 == r2) {
			return;
		}

		if (r2 instanceof Variable) {
			r1.union(r2);
			return;
		}

		if (r2 instanceof Function) {
			Function f = (Function) r2;

			r1.union(f);

			if (f.args.length != args.length) {
				Type.renameRootVariables();
				Errors.errorMessage("Cannot unify functions with different number of arguments");
				throw new NotUnifiableException();
			}

			returnType.unify(f.returnType);
			for (int i = 0; i < args.length; i++) {
				args[i].unify(f.args[i]);
			}
			return;
		}
		Type.renameRootVariables();
		Errors.errorMessage("Cannot unify '" + r1.toString() + "' with '"
				+ r2.toString() + "'");
		throw new NotUnifiableException();
	}

	public @Override
	String toString() {
		Function f = (Function) find();

		if (f.isMarked()) {
			return "...";
		}
		f.setMarked();

		String s = "(";
		for (int i = 0; i < f.args.length - 1; i++) {
			s += f.args[i].toString() + ",";
		}
		if (f.args.length != 0) {
			s += f.args[f.args.length - 1].toString();
		}

		s += ")->(";

		s += f.returnType.toString();
		s += ")";
		return s;
	}

}
