package tip.types;

import tip.*;

public class Pointer extends Type {
	private Type reftype;

	public Pointer(Type reftype) {
		this.reftype = reftype;
	}

	public Type getRefType() {
		return reftype;
	}

	@Override
	public void unify(Type other) {
		Type r1 = find();
		Type r2 = other.find();

		if (r1 == r2) {
			return;
		}

		if (r2 instanceof Variable) {
			r1.union(r2);
			return;
		}

		if (!(r2 instanceof Pointer)) {
			Type.renameRootVariables();
			Errors.errorMessage("Cannot unify '" + toString() + "' with '"
					+ other.toString() + "'");
			throw new NotUnifiableException();
		}

		Pointer p = (Pointer) r2;
		r1.union(p);

		// Unify child nodes
		reftype.unify(p.reftype);
	}

	public @Override
	String toString() {
		String s = "&";
		Type root = find();

		if (root.isMarked()) {
			return "...";
		}
		root.setMarked();
		return s + reftype.find().toString();
	}
}
